/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|          Program Name: Bin2C                                                                                          |
|                Author: Michael Krmer                                                                                 |
|                        Michael@dk8pp.de               (private)                                                       |
|                        Michael.Kraemer@eu.necel.com   (business)                                                      |
|                  Date: 13-Apr-2005                                                                                    |
|              Language: Open Watcom C 1.7                                                                              |
|               Version: 1.1                                                                                            |
|        Program status: Modified to use WinLib                                                                         |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Convert a binary file into C source code                                                                    |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <commctrl.h>
#include <ctype.h>
#include <time.h>
#include <mytype.h>
#include <wavelib.h>
#include "WinDef.h"
#include "WinLib.h"
#include "Bin2C.h"

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define STRINGSIZE 256          /* default size for strings */
#define PARMSIZE 20             /* maximum size of a string or positional command line parameter */
#define MAX_LOADSTRING 100
#define BUF_LEN 256             /* general buffer size */
#define INFILENAME_SIZE 50000   /* size for input filename buffer,
                                   multiple files can be selected ! */
#define RESIZE_X 0x1            /* resize control in x direction */
#define RESIZE_Y 0x2            /* resize control in y direction */
#define MOVE_X   0x4            /* move control in x direction */
#define MOVE_Y   0x8            /* move control in y direction */

#define MIN_SIZE_X 466          /* Minimum horizontal dialog size */
#define MIN_SIZE_Y 194          /* Minimum vertical dialog size */

/* headers and trailers for binary and wave files */
#define BIN_HEADER  "#pragma ghs section data=\".images\"\n"
#define BIN_TRAILER "#pragma ghs section data=default\n"
#define WAV_HEADER  "#pragma ghs section data=\".wave\"\n"
#define WAV_TRAILER "#pragma ghs section data=default\n"

#define MY_HKEY HKEY_CURRENT_USER // handle of registry key
#define MY_REG_KEY MY_REG_KEY_ROOT "Bin2C" // Registry key used by this program
#define PROGRAMNAME "Binary to C-Source Converter"
#define PURPOSE "This program converts binary files to C source code which\r\r\n"\
                "can then be compiled and embedded into the executable.\r\r\n"\
                "\r\r\n"
#define PROGRAMVERSION 110      /* will be displayed as n.nn */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
HINSTANCE       MyInstance ;
HWND            MainWindow ;
_BYTE strng1[STRINGSIZE] ;
_BYTE strng2[STRINGSIZE] ;
_BOOL sort, toc, dec_wav ;

/*
 Registry definitions
*/

_SWORD registry_x ;             /* left edge of dialog */
_SWORD registry_y ;             /* top edge of dialog */
_SWORD registry_width ;         /* width of dialog */
_SWORD registry_height ;        /* height of dialog */
_WORD registry_flags ;          /* store the flags */
_SWORD registry_version ;       /* version of program which wrote the registry */
_BYTE registry_filename[STRINGSIZE+1] ; /* filename for registry */
REGVALUE regvalue[] = {{"posx", REG_DWORD, &registry_x, 0},
                       {"posy", REG_DWORD, &registry_y, 0},
                       {"width", REG_DWORD, &registry_width, 0},
                       {"height", REG_DWORD, &registry_height, 0},
                       {"flags", REG_DWORD, &registry_flags, 0},
                       {"version", REG_DWORD, &registry_version, 0},
                       {"Filename", REG_SZ, &registry_filename, STRINGSIZE}} ;
MYREG myreg = {MY_HKEY, MY_REG_KEY, &regvalue[0], sizeof(regvalue)/sizeof(regvalue[0])} ;

/*
 Command line parameter definitions
*/
_BOOL disablereg = FALSE ;      /* disable use of the registry */
_BYTE posparm1[PARMSIZE+1] ;    /* positional parameter 1 */
_BYTE posparm2[PARMSIZE+1] ;    /* positional parameter 2 */
_BYTE strng[PARMSIZE+1] ;       /* string parameter strng */

CLO clo[] = {{"rc", PARM_SIMPLE, NULL, 0, FALSE, "clear registry entries for this program"},
             {"rd", PARM_SIMPLE, NULL, 0, FALSE, "disable use of the registry for this program"},
             {"pp", PARM_INT, NULL, 0, FALSE, "set process priority for dialog\r\r\n"
                                       "       (0:idle, 1:normal, 2:high, 3:realtime)"},
             {"pt", PARM_INT, NULL, 0, FALSE, "set priority for processing thread\r\r\n"
                                       "       (0:idle, 1:lowest, 2:below normal, 3:normal,\r\r\n"
                                       "        4:above normal, 5:highest, 6:time critical)"}} ;
_WORD clo_size = sizeof(clo)/sizeof(clo[0]) ;

/* These are indexes into the CLO table above. Keep them in sync */
#define PARM_RC 0
#define PARM_RD 1
#define PARM_PP 2
#define PARM_PT 3

_BYTE errorparm[MAXPARMLENGTH] ;

/*
 Dialog coordinates definitions
*/
COORDS coordinates[] = {{0, 0, 0, 0, 0, 0},  /* first entry is reserved for main dialog */
#ifdef SUSPENDBUTTON
                        {BUT_SUSPEND, 0, 0, 0, 0, MOVE_X|MOVE_Y},
#endif
                        {DLG_IN, 0, 0, 0, 0, RESIZE_X},
                        {BUT_IN, 0, 0, 0, 0, MOVE_X},
                        {DLG_OUT, 0, 0, 0, 0, RESIZE_X},
                        {BUT_OUT, 0, 0, 0, 0, MOVE_X},
                        {BUT_START, 0, 0, 0, 0, MOVE_X|MOVE_Y},
                        {BUT_EXIT, 0, 0, 0, 0, MOVE_X|MOVE_Y},
                        {BUT_INFO, 0, 0, 0, 0, MOVE_X|MOVE_Y},
                        {DLG_STATUS, 0, 0, 0, 0, RESIZE_X|RESIZE_Y|MOVE_Y}
                       } ;
COORDS *sorted_coord[sizeof(coordinates)/sizeof(coordinates[0])-1] ;
_WORD coord_size = sizeof(coordinates)/sizeof(coordinates[0]) ;
_BOOL validreg = FALSE ;        /* valid data from registry retrieved */
_BOOL abort_thread = FALSE ; /* set TRUE to abort the thread */
_BOOL thread_suspended = FALSE ;
_WORD volatile vol ;
_WORD volatile vol1 ;
_BYTE purpose[] = PURPOSE ;
_WORD mult_files = 0 ; /* number of files if multiple files specified, 0 if just one file */
_BYTE infilename[INFILENAME_SIZE] ; /* statically allocated!!! */
_BYTE infilename_no_path[256] ;
_BYTE outfilename[256] = "" ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/
BOOL CALLBACK Bin2C(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam ) ;
int sortieren(const void *a, const void *b) ;
_BOOL output_binary(_BYTE *outfilename, _BYTE *path, _BYTE *fn_nopath,
                    _BYTE **fnames, _WORD number_of_files) ;
_BOOL output_wave(_BYTE *outfilename, _BYTE *path, _BYTE *fn_nopath,
                  _BYTE **fnames, _WORD number_of_files) ;
void make_varname(_BYTE *varname, _BYTE *filename) ;
DWORD WINAPI DoOperation(LPVOID lpParam) ;
void EnableControls(_BOOL enable) ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       WinMain                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| MAIN program.                                                                                                         |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int WINAPI WinMain(HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline, int cmdshow)
{
_LONG static volatile rv ;
_BOOL startup ;
   startup = TRUE ;
   errorparm[0] = '\0' ;
   getparms(cmdline, errorparm) ;
   if (clo[PARM_RC].found) clear_registry(&myreg) ;
   disablereg = clo[PARM_RD].found ;
   if (startup)
   { /* start windowed version */
      if (clo[PARM_PP].found) set_process_prio(clo[PARM_PP].value) ;
      MyInstance = this_inst ;
      InitCommonControls() ;
      rv = DialogBox(this_inst, "DLG_BIN2C", 0, (DLGPROC)Bin2C) ;
   }
   return(0);
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Bin2C                                                                                                           |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Processes messages for the DLG_BIN2C dialog                                                                           |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
BOOL CALLBACK Bin2C(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam )
{
OPENFILENAME static infilestruct ;
OPENFILENAME static outfilestruct ;

_BYTE static def_inext[] = "Graphics files\0*.png;*.bmp;*.jpg\0"
                           "Audio files\0*.mp2;*.mpa;*.mp3;*.wav\0"
                           "Video files\0*.mpv;*.mpg;*.avi;*.mpeg\0"
                           "All files\0*.*\0" ;

_BYTE static outfilename_no_path[256] ;
_BYTE static def_outext[] = "C Source files\0*.c;*.cpp\0"
                            "All files\0*.*\0" ;

_BYTE static buf[BUF_LEN] ;

_WORD i ;
BOOL retcd ;
DWORD static dwThreadId, dwThrdParam = 1 ;
HANDLE static hThread ;
RECT static rect ;

_BYTE *out_msg ;

   retcd = 0 ; /* return 0 if we don't process the message */
   validreg = FALSE ;
   switch(msg)
   {
   case WM_INITDIALOG:
      retcd = 1 ;
      MainWindow = window_handle ;
// Make sure to define the below mentioned icon!
      SendMessage(MainWindow, WM_SETICON, ICON_BIG, (LPARAM) LoadIcon(MyInstance, "MyWinIcon")) ;
//    ShowError(DLG_STATUS) ; /* display last error message */

// save original coordinates of dialog
      SaveCoordinates() ;

      validreg = get_registry(&myreg) ; // get registry content

      if (validreg)
      {
         MoveWindow(
         MainWindow,            // handle of window
         registry_x,            // horizontal position
         registry_y,            // vertical position
         registry_width,        // width
         registry_height,       // height
         TRUE);                 // repaint flag
         CheckDlgButton(MainWindow, CB_SORT,   ((registry_flags & 0x00000001) == 0) ? BST_UNCHECKED : BST_CHECKED) ;
         CheckDlgButton(MainWindow, CB_TOC,    ((registry_flags & 0x00000002) == 0) ? BST_UNCHECKED : BST_CHECKED) ;
         CheckDlgButton(MainWindow, CB_DECWAV, ((registry_flags & 0x00000004) == 0) ? BST_UNCHECKED : BST_CHECKED) ;
      }
      else
      {
         CheckDlgButton(MainWindow,CB_SORT,BST_CHECKED) ;
         CheckDlgButton(MainWindow,CB_TOC ,BST_CHECKED) ;
      }

      /* display any illegal parameter */
      if (strlen(errorparm) != 0)
      {
         LoadString(MyInstance, STR_INVALIDPARM, strng1, STRINGSIZE);
         sprintf(strng2, strng1, errorparm) ;
         SetDlgItemText(MainWindow, DLG_STATUS, strng2) ;
      }
#ifdef SUSPENDBUTTON
      EnableWindow(GetDlgItem(MainWindow, BUT_SUSPEND), FALSE) ;
#endif
   break ;

   case WM_USER:
      retcd = 1 ;
      switch(LOWORD(wparam))
      {
      case USER_THREAD_ENDED:
      // output the results
         LoadString(MyInstance, STR_NORMAL_END, strng1, STRINGSIZE) ;
         out_msg = strng1 ;
/* Thread may have assembled a message. If so, use it */
         if (lparam != NULL) out_msg = (_BYTE *) lparam ;
         SetDlgItemText(MainWindow, DLG_STATUS, out_msg) ;
         CloseHandle(hThread) ;
         hThread = NULL ;
         EnableControls(TRUE) ;
#ifdef SUSPENDBUTTON
         EnableWindow(GetDlgItem(MainWindow, BUT_SUSPEND), FALSE) ;
#endif
      break ;
      }
   break ;

   case WM_COMMAND:
      retcd = 1 ;
      switch( LOWORD( wparam ) )
      {
      case BUT_IN:
         infilename[0] = '\0' ;
         infilestruct.lStructSize = sizeof(OPENFILENAME) ;
         infilestruct.hwndOwner = MainWindow ;
         infilestruct.hInstance = 0 ;
         infilestruct.lpstrFilter = def_inext ;
         infilestruct.lpstrCustomFilter = NULL ;
         infilestruct.nMaxCustFilter = 0 ;
         infilestruct.nFilterIndex = 1 ;
         infilestruct.lpstrFile = infilename ;
         infilestruct.nMaxFile = INFILENAME_SIZE ;
         infilestruct.lpstrFileTitle = infilename_no_path ;
         infilestruct.nMaxFileTitle = sizeof(infilename_no_path) ;
         infilestruct.lpstrInitialDir = NULL ;//"d:\Development" ;
         infilestruct.lpstrTitle = "Binary input file(s)" ;
         infilestruct.Flags = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY
                            | OFN_ALLOWMULTISELECT | OFN_EXPLORER ;
         infilestruct.nFileOffset = 0 ;
         infilestruct.nFileExtension = 0 ;
         infilestruct.lpstrDefExt = NULL ;
         infilestruct.lCustData = NULL ;
         infilestruct.lpfnHook = NULL ;
         infilestruct.lpTemplateName = NULL ;
         if (GetOpenFileName(&infilestruct) != 0)
         {
            i = 0 ;
            mult_files = 0 ;
            while ((infilestruct.lpstrFile[i+0] != '\0')
                && (infilestruct.lpstrFile[i+1] != '\0'))
            {
               while (infilestruct.lpstrFile[i] != '\0') i++ ;
               if (infilestruct.lpstrFile[i+1] != '\0') {i++ ; mult_files++ ;}
            }
            if (mult_files == 0)
               SetDlgItemText(MainWindow, DLG_IN, infilestruct.lpstrFile) ;
            else
            {
               sprintf(buf, "%s (%u files selected)", infilestruct.lpstrFile, mult_files) ;
               SetDlgItemText(MainWindow, DLG_IN, buf) ;
            }
         }
      break ;

      case DLG_IN:
         if (mult_files == 0)
            GetDlgItemText(MainWindow, DLG_IN, infilename, INFILENAME_SIZE) ;
      break ;

      case BUT_OUT:
         outfilestruct.lStructSize = sizeof(OPENFILENAME) ;
         outfilestruct.hwndOwner = MainWindow ;
         outfilestruct.hInstance = 0 ;
         outfilestruct.lpstrFilter = def_outext ;
         outfilestruct.lpstrCustomFilter = NULL ;
         outfilestruct.nMaxCustFilter = 0 ;
         outfilestruct.nFilterIndex = 1 ;
         outfilestruct.lpstrFile = outfilename ;
         outfilestruct.nMaxFile = sizeof(outfilename) ;
         outfilestruct.lpstrFileTitle = outfilename_no_path ;
         outfilestruct.nMaxFileTitle = sizeof(outfilename_no_path) ;
         outfilestruct.lpstrInitialDir = NULL ;//"d:\Development" ;
         outfilestruct.lpstrTitle = "C Source output file" ;
         outfilestruct.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT ;
         outfilestruct.nFileOffset = 0 ;
         outfilestruct.nFileExtension = 0 ;
         outfilestruct.lpstrDefExt = NULL ;
         outfilestruct.lCustData = NULL ;
         outfilestruct.lpfnHook = NULL ;
         outfilestruct.lpTemplateName = NULL ;
         if (GetSaveFileName(&outfilestruct) != 0)
         {
            SetDlgItemText(MainWindow, DLG_OUT, outfilestruct.lpstrFile) ;
         }
      break ;

      case DLG_OUT:
         GetDlgItemText(MainWindow, DLG_OUT, outfilename, sizeof(outfilename)) ;
      break ;

      case BUT_INFO:
         sprintf(strng1, "Version %u.%2.2u", PROGRAMVERSION/100, PROGRAMVERSION%100) ;
         PrintProgramInfo(DLG_HELPLIST, PROGRAMNAME, strng1, __DATE__, __TIME__) ;
      break ;

      case IDOK:
      case BUT_START:
         /* The start button becomes abort, if the thread is suspended */
         if (thread_suspended)
         {
            abort_thread = TRUE ;
#ifdef SUSPENDBUTTON
            SendMessage(MainWindow, WM_COMMAND, BUT_SUSPEND, 0) ;
#endif
         }
         else
         {
/* Operation is done in a thread. Start it. */
            hThread = CreateThread(
                NULL,                        // no security attributes
                0,                           // use default stack size
                DoOperation,                 // thread function
                &dwThrdParam,                // argument to thread function
                0,                           // use default creation flags
                &dwThreadId);                // returns the thread identifier

        // Check the return value for success.
            if (hThread == NULL)
               LoadString(MyInstance, STR_THREADSTARTERROR, strng1, STRINGSIZE) ;
            else
            { /* Thread started properly */
               LoadString(MyInstance, STR_WORKING, strng1, STRINGSIZE) ;
               EnableControls(FALSE) ;
               thread_suspended = FALSE ;
#ifdef SUSPENDBUTTON
               EnableWindow(GetDlgItem(MainWindow, BUT_SUSPEND), TRUE) ;
#endif
               if (clo[PARM_PT].found) set_thread_prio(hThread, clo[PARM_PT].value) ;
            }
            SetDlgItemText(MainWindow, DLG_STATUS, strng1) ;
         }
      break ;

      case IDABORT:
      case IDCANCEL:
      case BUT_EXIT:
         GetWindowRect(MainWindow, &rect) ; /* get current dialog coordinates */
         registry_x = rect.left ;
         registry_y = rect.top ;
         registry_width = rect.right - rect.left ;
         registry_height = rect.bottom - rect.top ;
         if (registry_x < 0) registry_x = 0 ;
         if (registry_y < 0) registry_y = 0 ;
         registry_flags  = (IsDlgButtonChecked(MainWindow, CB_SORT) == BST_CHECKED) ?   0x00000001 : 0 ;
         registry_flags |= (IsDlgButtonChecked(MainWindow, CB_TOC) == BST_CHECKED) ?    0x00000002 : 0 ;
         registry_flags |= (IsDlgButtonChecked(MainWindow, CB_DECWAV) == BST_CHECKED) ? 0x00000004 : 0 ;
         registry_version = PROGRAMVERSION ;    /* version of program which wrote the registry */
         put_registry(&myreg) ;                 /* store the dialog properties in registry */
         EndDialog(MainWindow, TRUE) ;          /* exits the dialog box */
      break ;

      default:
         retcd = 0 ; /* we have not processed this command, so return zero */
      break ;
      }
   break ;

   default:
      retcd = WinLibMessage(msg, wparam, lparam) ; /* WinLib processes some messages */
   break ;
   }

   return(retcd);
} /* Bin2C */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       void EnableControls(_BOOL enable)                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Enable or disable some critical controls during execution of thread.                                                  |
| Control gets enabled if enable is TRUE, otherwise it gets disabled.                                                   |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void EnableControls(_BOOL enable)
{
   EnableWindow(GetDlgItem(MainWindow, BUT_START), enable) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       DoOperation(LPVOID lpParam)                                                                                     |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| This thread does whatever this program is supposed to do.                                                             |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
DWORD WINAPI DoOperation(LPVOID lpParam)
{
_BYTE static strng_thread1[STRINGSIZE] ;
_BYTE *lpMsgBuf ;
_BYTE *retcd ;
_BOOL err ;
_WORD i, j ;
_BYTE static **fnames ;    /* array of pointers to filenames */

   retcd = NULL ; /* NULL means no error */

   sort = IsDlgButtonChecked(MainWindow,CB_SORT) ;
   toc = IsDlgButtonChecked(MainWindow,CB_TOC) ;
   dec_wav = IsDlgButtonChecked(MainWindow,CB_DECWAV) ;
   err = FALSE ;
   // clear status display
   LoadString(MyInstance, STR_EMPTY, strng, MAX_LOADSTRING);
   SetDlgItemText(MainWindow, DLG_STATUS, strng) ;

/* collect pointers to filenames */
   if (mult_files != 0)
   {
      fnames = (_BYTE **) malloc((mult_files)*sizeof(_BYTE *)) ;
      if (fnames == NULL)
      {
         LoadString(MyInstance, STR_MALLOC_FAILED, strng, MAX_LOADSTRING);
         SetDlgItemText(MainWindow, DLG_STATUS, strng) ;
      }
      j = 0 ;
      for (i=0; i<mult_files; i++)
      {
         while (infilename[j] != '\0') j++ ;
         fnames[i] = &infilename[j+1] ;
         j++ ;
      }
      if (sort)
      { /* sort filenames */
         qsort(fnames,
               mult_files,
               sizeof(_BYTE *),
               sortieren) ;
      }
   }

   if (!dec_wav) err = output_binary(outfilename, infilename, infilename_no_path, fnames, mult_files) ;
   else          err = output_wave(outfilename, infilename, infilename_no_path, fnames, mult_files) ;

   if (!err)
      LoadString(MyInstance, STR_DONE, strng_thread1, MAX_LOADSTRING);
   else
   {
      FormatMessage(
          FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
          NULL,
          GetLastError(),
          MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
          (LPTSTR) &lpMsgBuf,
          0,
          NULL) ;
          strcpy(strng_thread1, lpMsgBuf) ;
      // Free the buffer.
          LocalFree(lpMsgBuf) ;
   }

   retcd = strng_thread1 ;
   if (fnames != NULL) free(fnames) ;

// Notify main window that the thread is finished
   SendMessage(MainWindow, WM_USER, USER_THREAD_ENDED, (LPARAM) retcd) ;

   return 0;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       int sortieren(const void *a, const void *b)                                                                     |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| For quicksort                                                                                                         |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int sortieren(const void *a, const void *b)
{
   return(stricmp(*((const char **) a), *((const char **) b))) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       _BOOL output_wave(_BYTE *outfilename, _BYTE *path, _BYTE *fn_nopath, _BYTE **fnames, _WORD number_of_files)     |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Decode wave file                                                                                                      |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_BOOL output_wave(_BYTE *outfilename, _BYTE *path, _BYTE *fn_nopath, _BYTE **fnames, _WORD number_of_files)
{
WFILE *infile ;
FILE *outfile ;
_BOOL err ;
_WORD filenr ;
_BYTE static next_filename[256] = "" ;
_BYTE static varname[256] = "" ;
_WORD i, j ;
_BYTE *fn ;
_SSHORT sample ;

   err = FALSE ;
// open output file
   outfile = fopen(outfilename,"wt") ;
   if (outfile == NULL) err = TRUE ;

   filenr = 0 ;
   if (!err)
   {
      fprintf(outfile, WAV_HEADER) ;
      do
      {
         if (number_of_files == 0)
            strcpy(next_filename, path) ;
         else
         {
            sprintf(next_filename,"%s\\%s",path ,fnames[filenr]) ;
         }
// open next input file
         infile = wfopen(next_filename) ;

         if (infile == NULL) err = TRUE ;

         if (!err)
         {
            fprintf(outfile, "signed short ") ;
            i = 0 ;
            if (number_of_files == 0)
               fn = fn_nopath ;
            else
               fn = fnames[filenr] ;
            make_varname(&varname, fn) ;
            fprintf(outfile, "%s[] = {", varname) ;

            i = 0 ;
            while (!wfeof(infile))
            {
               if (wfgetsample(&sample, infile) != NULL)
               {
                  if (i > 0) fprintf(outfile,",") ;
                  if ((i % 12) != 0)
                     fprintf(outfile," ") ;
                  else
                     fprintf(outfile,"\n        ") ;
                  fprintf(outfile,"%6.1i", sample) ;
                  i++ ;
               }
            }

            fprintf(outfile, "} ;\n\n") ;
            fprintf(outfile,"unsigned long %s_size = sizeof(%s)/sizeof(signed short) ;\n", varname, varname) ;
            fprintf(outfile,"unsigned short %s_BitsPerSample = %u ;\n", varname, infile->BitsPerSample) ;
            fprintf(outfile,"unsigned short %s_Channels = %u ;\n", varname, infile->Channels) ;
            fprintf(outfile,"unsigned long %s_SamplesPerSecond = %u ;\n\n", varname, infile->SamplesPerSecond) ;
            wfclose(infile) ;
            filenr++ ;
         }
      }
      while ((!err) && (filenr < number_of_files)) ;
   }

   if ((!err) && toc)
   {
      fprintf(outfile, "signed short *toc[] ={\n") ;
      for (i=0; i<filenr; i++)
      {
         j = 0 ;
         if (i != 0) fprintf(outfile, ",\n") ;
         fprintf(outfile,"        ") ;
         if (number_of_files == 0)
            fn = fn_nopath ;
         else
            fn = fnames[i] ;
         make_varname(&varname, fn) ;
         fprintf(outfile, "%s", varname) ;
      }
      fprintf(outfile, "} ;\n") ;
      fprintf(outfile, "int toc_size = sizeof(toc)/sizeof(signed short *) \;\n") ;
   }

   if (!err)
      fprintf(outfile, WAV_TRAILER) ;
   if (outfile != NULL) fclose(outfile) ;
   return(err) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       _BOOL output_binary(_BYTE *outfilename, _BYTE *path, _BYTE *fn_nopath, _BYTE **fnames, _WORD number_of_files)   |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Decode wave file                                                                                                      |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_BOOL output_binary(_BYTE *outfilename, _BYTE *path, _BYTE *fn_nopath, _BYTE **fnames, _WORD number_of_files)
{
FILE *infile ;
FILE *outfile ;
_BOOL err ;
_WORD filenr ;
_BYTE static next_filename[256] = "" ;
_BYTE static varname[256] = "" ;
_WORD i, j ;
_BYTE *fn ;
_BYTE next_char ;

   err = FALSE ;
// open output file
   outfile = fopen(outfilename,"wt") ;
   if (outfile == NULL) err = TRUE ;

   filenr = 0 ;
   if (!err)
   {
      fprintf(outfile, BIN_HEADER) ;
      do
      {
         if (number_of_files == 0)
            strcpy(next_filename, path) ;
         else
         {
            sprintf(next_filename,"%s\\%s",path ,fnames[filenr]) ;
         }
// open next input file
         infile = fopen(next_filename, "rb") ;

         if (infile == NULL) err = TRUE ;

         if (!err)
         {
            fprintf(outfile, "unsigned char ") ;
            i = 0 ;
            if (number_of_files == 0)
               fn = fn_nopath ;
            else
               fn = fnames[filenr] ;
            make_varname(&varname, fn) ;
            fprintf(outfile, "%s[] = {", varname) ;

            i = 0 ;
            while (!feof(infile))
            {
               if (fread(&next_char,1,1,infile) == 1)
               {
                  if (i > 0) fprintf(outfile,",") ;
                  if ((i % 16) != 0)
                     fprintf(outfile," ") ;
                  else
                     fprintf(outfile,"\n        ") ;
                  fprintf(outfile,"0x%2.2x", next_char) ;
                  i++ ;
               }
            }

            fprintf(outfile, "} ;\n\n") ;
            fclose(infile) ;
            filenr++ ;
         }
      }
      while ((!err) && (filenr < number_of_files)) ;
   }

   if ((!err) && toc)
   {
      fprintf(outfile, "unsigned char *toc[] ={\n") ;
      for (i=0; i<filenr; i++)
      {
         j = 0 ;
         if (i != 0) fprintf(outfile, ",\n") ;
         fprintf(outfile,"        ") ;
         if (number_of_files == 0)
            fn = fn_nopath ;
         else
            fn = fnames[i] ;
         make_varname(&varname, fn) ;
         fprintf(outfile, "%s", varname) ;
      }
      fprintf(outfile, "} ;\n") ;
      fprintf(outfile, "int toc_size = sizeof(toc)/sizeof(unsigned char *) \;\n") ;
   }

   if (!err)
      fprintf(outfile, BIN_TRAILER) ;
   if (outfile != NULL) fclose(outfile) ;
   return(err) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       void make_varname(_BYTE *varname, _BYTE *filename)                                                              |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Make valid variable name from filename. Remove special characters.                                                    |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void make_varname(_BYTE *varname, _BYTE *filename)
{
_WORD i, j ;
   i = 0 ; j = 0 ;
   while ((filename[i] != '.') && (filename[i] != '\0'))
   {
      if ( ((i == 0) && isalpha(filename[i]))
         ||((i != 0) && isalnum(filename[i])) )
      {
         varname[j] = filename[i] ;
         j++ ;
      }
      i++ ;
   }
   varname[j] = '\0' ;
}


