/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|          Program Name: ExecuteAt                                                                                      |
|                Author: Michael Krmer                                                                                 |
|                        Michael@dk8pp.de               (private)                                                       |
|                        Michael.Kraemer@eu.necel.com   (business)                                                      |
|                  Date: 16-Aug-2006                                                                                    |
|              Language: Open Watcom C 1.7                                                                              |
|               Version: 0.2                                                                                            |
|        Program status: still testing                                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Start a program at a specific time and day of the week                                                      |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include <time.h>
#include "WinDef.h"
#include "mytype.h"
#include "WinLib.h"
#include "ExecuteAt.h"

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define STRINGSIZE 256          /* default size for strings */
#define PARMSIZE 20             /* maximum size of a string or positional command line parameter */
//#define SUSPENDBUTTON           /* define if a suspend button shall be supported */

#define MY_HKEY HKEY_CURRENT_USER // handle of registry key
#define MY_REG_KEY MY_REG_KEY_ROOT "ExecuteAt" // Registry key used by this program
#define PROGRAMNAME "Execute at"
#define PURPOSE "This program executes another program at a specified time and day.\r\r\n"\
                "\r\r\n"
#define PROGRAMVERSION 20       /* will be displayed as n.nn */

#define MAX_CMDLINE     256     /* maximum length of a commandline */
#define FLAGS_SIZE      16      /* size of flags */
#define HISTORY_SIZE    10      /* maximum number of entries in the history list */
#define REG_HISTORY_SIZE ((HISTORY_SIZE * MAX_CMDLINE) + HISTORY_SIZE + 1) /* Size of history in registry */
#define HISTORY_ITEMS   3       /* three items per entry: command name, flags, time */

#define SLEEPTIME       200     /* Sleep time in ms for timer thread */

#define TRIGGER_MARGIN  60      /* Margin for the trigger time. Program will be started up to TRIGGER_MARGIN
                                   seconds after the defined trigger time */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
HINSTANCE       MyInstance ;
HWND            MainWindow ;
_BYTE strng1[STRINGSIZE] ;
_BYTE strng2[STRINGSIZE] ;
/*
 Registry definitions
*/
_SWORD registry_x ;             /* left edge of dialog */
_SWORD registry_y ;             /* top edge of dialog */
_SWORD registry_width ;         /* width of dialog */
_SWORD registry_height ;        /* height of dialog */
_SWORD registry_version ;       /* version of program which wrote the registry */
_BYTE registry_history[REG_HISTORY_SIZE] ; /* history */
REGVALUE regvalue[] = {{"posx", REG_DWORD, &registry_x, 0},
                       {"posy", REG_DWORD, &registry_y, 0},
                       {"width", REG_DWORD, &registry_width, 0},
                       {"height", REG_DWORD, &registry_height, 0},
                       {"version", REG_DWORD, &registry_version, 0},
                       {"history", REG_MULTI_SZ, &registry_history, REG_HISTORY_SIZE} } ;
MYREG myreg = {MY_HKEY, MY_REG_KEY, &regvalue[0], sizeof(regvalue)/sizeof(regvalue[0])} ;

/*
 Command line parameter definitions
*/
_BOOL disablereg = FALSE ;      /* disable use of the registry */

CLO clo[] = {{"rc", PARM_SIMPLE, NULL, 0, FALSE, "clear registry entries for this program"},
             {"rd", PARM_SIMPLE, NULL, 0, FALSE, "disable use of the registry for this program"},
             {"pp", PARM_INT, NULL, 0, FALSE, "set process priority for dialog\r\r\n"
                                       "       (0:idle, 1:normal, 2:high, 3:realtime)"},
             {"pt", PARM_INT, NULL, 0, FALSE, "set priority for processing thread\r\r\n"
                                       "       (0:idle, 1:lowest, 2:below normal, 3:normal,\r\r\n"
                                "        4:above normal, 5:highest, 6:time critical)"} } ;
_WORD clo_size = sizeof(clo)/sizeof(clo[0]) ;

/* These are indexes into the CLO table above. Keep them in sync */
#define PARM_RC 0
#define PARM_RD 1
#define PARM_PP 2
#define PARM_PT 3

_BYTE errorparm[MAXPARMLENGTH] ;

/*
 Dialog coordinates definitions
*/
COORDS coordinates[] = {{0, 0, 0, 0, 0, 0},  /* first entry is reserved for main dialog */
#ifdef SUSPENDBUTTON
                        {BUT_SUSPEND, 0, 0, 0, 0, MOVE_X|MOVE_Y},
#endif
                        {DLG_STATUS, 0, 0, 0, 0, RESIZE_X|RESIZE_Y|MOVE_Y},
                        {DLG_EXECUTEFILE, 0, 0, 0, 0, RESIZE_X},
                        {DLG_AT, 0, 0, 0, 0, RESIZE_X},
                        {BUT_INFO, 0, 0, 0, 0, 0},
                        {BUT_START, 0, 0, 0, 0, MOVE_X},
                        {BUT_CANCEL, 0, 0, 0, 0, MOVE_X},
                        {BUT_EXIT, 0, 0, 0, 0, MOVE_X}
                       } ;
COORDS *sorted_coord[sizeof(coordinates)/sizeof(coordinates[0])-1] ;
_WORD coord_size = sizeof(coordinates)/sizeof(coordinates[0]) ;
_BOOL validreg = FALSE ;        /* valid data from registry retrieved */
_BOOL abort_thread = FALSE ; /* set TRUE to abort the thread */
_BOOL thread_suspended = FALSE ;
_WORD volatile vol ;
_WORD volatile vol1 ;
_BYTE purpose[] = PURPOSE ;

_BYTE command[MAX_CMDLINE] ;
_WORD volatile start_time ;     // start time in seconds since midnight
_BYTE *phistory[HISTORY_SIZE] ;
_BYTE history[HISTORY_SIZE][MAX_CMDLINE] ;
_BYTE flags[FLAGS_SIZE] ;
_WORD oldindex ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/
BOOL CALLBACK ExecuteAt(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam) ;
void display_info(_BYTE *msg) ;
DWORD WINAPI WaitExecute(LPVOID lpParam) ;
void get_history(_BYTE **phistory, _WORD size) ;
void put_history(_BYTE **phistory, _WORD size, _BYTE *newcommand, _BYTE *time, _BYTE *flags) ;
void init_combobox(void) ;
void init_dialog(_BYTE *strng) ;
void end_program(void) ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       WinMain                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| MAIN program.                                                                                                         |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int WINAPI WinMain(HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline, int cmdshow)
{
_LONG static volatile rv ;
_BOOL startup ;
_WORD i ;
   startup = TRUE ;
   errorparm[0] = '\0' ;
   getparms(cmdline, errorparm) ;
   if (clo[PARM_RC].found) clear_registry(&myreg) ;
   disablereg = clo[PARM_RD].found ;
   for (i=0; i<HISTORY_SIZE; i++)
   {
      phistory[i] = history[i] ;
      history[i][0] = '\0' ;
      history[i][1] = '\0' ;
   }
   oldindex = -1 ;
   if (startup)
   { /* start windowed version */
      if (clo[PARM_PP].found) set_process_prio(clo[PARM_PP].value) ;
      MyInstance = this_inst ;
      InitCommonControls() ;
      rv = DialogBox(this_inst, "DLG_EXECUTEAT", 0, (DLGPROC)ExecuteAt) ;
   }
   return(0);
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       ExecuteAt                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Processes messages for the DLG_EXECUTEAT dialog                                                                       |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
BOOL CALLBACK ExecuteAt(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam)
{
OPENFILENAME execfilestruct ;

//FILE *outfile ;
_BYTE static exefilename[256] = "" ;
_BYTE static exefilename_no_path[256] ;
_BYTE static def_out_ext[] = "Executable files\0*.exe;*.bat;*.cmd\0All files\0*.*\0\0" ;
DWORD static dwThreadId, dwThrdParam = 1;
HANDLE static hThread = NULL ;
BOOL retcd ;
BOOL static terminate_program ;
RECT static rect ;
HFONT hFont ;
_BYTE str1[16], str2[16], str3[16] ;
_WORD len, len1 ;
_WORD i ;
_WORD start_hour ;
_WORD start_minute ;
_WORD start_second ;

   retcd = 0 ; /* return 0 if we don't process the message */

   switch(msg)
   {
   case WM_INITDIALOG:
      retcd = 1 ;
      MainWindow = window_handle ;
// Make sure to define the below mentioned icon!
      SendMessage(MainWindow, WM_SETICON, ICON_BIG, (LPARAM) LoadIcon(MyInstance, "MyWinIcon")) ;
//    ShowError(DLG_STATUS) ; /* display last error message */

// save original coordinates of dialog
      SaveCoordinates() ;

// Select font for controls (sample code)
      hFont = CreateFont(
          15,                  // logical height of font
          0,                   // logical average character width
          0,                   // angle of escapement
          0,                   // base-line orientation angle
          FW_NORMAL,           // font weight
          FALSE,               // italic attribute flag
          FALSE,               // underline attribute flag
          FALSE,               // strikeout attribute flag
          ANSI_CHARSET,        // character set identifier
          OUT_DEFAULT_PRECIS,  // output precision
          CLIP_DEFAULT_PRECIS, // clipping precision
          DEFAULT_QUALITY,     // output quality
          DEFAULT_PITCH,       // pitch and family
          "Courier New"        // pointer to typeface name string
      ) ;

      SendMessage(GetDlgItem(MainWindow, DLG_EXECUTEFILE), WM_SETFONT, (WPARAM) hFont, MAKELPARAM(FALSE, 0));

      validreg = get_registry(&myreg) ; // get registry content

      if (validreg)
      {
         MoveWindow(
         MainWindow,            // handle of window
         registry_x,            // horizontal position
         registry_y,            // vertical position
         registry_width,        // width
         registry_height,       // height
         TRUE);                 // repaint flag
// Initialize history list
         init_combobox() ;
      }

      /* display any illegal parameter */
      if (strlen(errorparm) != 0)
      {
         LoadString(MyInstance, STR_INVALIDPARM, strng1, STRINGSIZE);
         sprintf(strng2, strng1, errorparm) ;
         SetDlgItemText(MainWindow, DLG_STATUS, strng2) ;
      }
#ifdef SUSPENDBUTTON
      EnableWindow(GetDlgItem(MainWindow, BUT_SUSPEND), FALSE) ;
#endif
      terminate_program = FALSE ;
   break ;

   case WM_USER:
      retcd = 1 ;
      switch(LOWORD(wparam))
      {
      case USER_THREAD_ENDED:
         LoadString(MyInstance, STR_THREADENDED, strng1, sizeof(strng1));
         SetDlgItemText(MainWindow, DLG_STATUS, strng1) ;
         CloseHandle(hThread);
         hThread = NULL ;
//         if (terminate_program) end_program() ;
         EnableWindow(GetDlgItem(MainWindow, BUT_EXECUTE), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, BUT_START), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, BUT_AT), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, DLG_EXECUTEFILE), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, DLG_AT), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, CB_KEEPOPEN), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, CB_SU), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, CB_MO), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, CB_TU), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, CB_WE), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, CB_TH), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, CB_FR), TRUE) ;
         EnableWindow(GetDlgItem(MainWindow, CB_SA), TRUE) ;
      break ;
      }
   break ;

   case WM_COMMAND:
      retcd = 1 ;
      switch(LOWORD(wparam))
      {
      case BUT_EXECUTE:
         execfilestruct.lStructSize = sizeof(OPENFILENAME) ;
         execfilestruct.hwndOwner = MainWindow ;
         execfilestruct.hInstance = 0 ;
         execfilestruct.lpstrFilter = def_out_ext ;
         execfilestruct.lpstrCustomFilter = NULL ;
         execfilestruct.nMaxCustFilter = 0 ;
         execfilestruct.nFilterIndex = 1 ;
         execfilestruct.lpstrFile = exefilename ;
         execfilestruct.nMaxFile = sizeof(exefilename) ;
         execfilestruct.lpstrFileTitle = exefilename_no_path ;
         execfilestruct.nMaxFileTitle = sizeof(exefilename_no_path) ;
         execfilestruct.lpstrInitialDir = NULL ;
         execfilestruct.lpstrTitle = "Sample data output file" ;
         execfilestruct.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT ;
         execfilestruct.nFileOffset = 0 ;
         execfilestruct.nFileExtension = 0 ;
         execfilestruct.lpstrDefExt = NULL ;
         execfilestruct.lCustData = NULL ;
         execfilestruct.lpfnHook = NULL ;
         execfilestruct.lpTemplateName = NULL ;
         if (GetOpenFileName(&execfilestruct) != 0)
         {
            SetDlgItemText(MainWindow, DLG_EXECUTEFILE, execfilestruct.lpstrFile) ;
         }
      break ;

      case DLG_EXECUTEFILE:
         GetDlgItemText(MainWindow, DLG_EXECUTEFILE, exefilename, sizeof(exefilename)) ;
         i = SendMessage(GetDlgItem(MainWindow, DLG_EXECUTEFILE), CB_GETCURSEL, 0, 0) ; // get selected entry
         if (i != CB_ERR)
         {
            if ((oldindex == -1) || (oldindex != i)) init_dialog(phistory[i]) ;
            oldindex = i ;
         }
      break ;

      case BUT_INFO:
         sprintf(strng1, "Version %u.%2.2u", PROGRAMVERSION/100, PROGRAMVERSION%100) ;
         PrintProgramInfo(DLG_HELPLIST, PROGRAMNAME, strng1, __DATE__, __TIME__) ;
      break ;

      case BUT_AT:
         MessageBox(MainWindow,
                    "Enter time as HH:MM:SS\r"
                    "   Example 14:53:27\r",
                    "Help", MB_OK) ;
      break ;

      case IDOK:
      case BUT_START:
/* Start */
/* Clear status display */
         abort_thread = FALSE ;
         LoadString(MyInstance, STR_EMPTY, strng1, sizeof(strng1));
         SetDlgItemText(MainWindow, DLG_STATUS, strng1) ;

         GetDlgItemText(MainWindow, DLG_EXECUTEFILE, exefilename, sizeof(exefilename)) ;

         if (IsDlgButtonChecked(MainWindow, CB_KEEPOPEN) == BST_CHECKED)
            strcpy(command,"cmd.exe /k \"") ;
         else
            strcpy(command,"cmd.exe /c \"") ;
         strcat(command, exefilename) ;
         strcat(command, "\"") ;

         GetDlgItemText(MainWindow, DLG_AT, strng1, sizeof(strng1)) ;
         start_hour = start_minute = start_second = 0 ;
         str1[0]='\0' ; str2[0]='\0' ; str3[0]='\0' ;
         len = sscanf(strng1, "%[^:.,;\n]%*1s%[^:.,;\n]%*1s%[^:.,;\n]", &str1, &str2, &str3) ;
         len = 0 ;
         len1 = sscanf(str1, "%d", &start_hour) ;
         if (len1 != EOF) len = len1 ;
         len1 = sscanf(str2, "%d", &start_minute) ;
         if (len1 != EOF) len += len1 ;
         len1 = sscanf(str3, "%d", &start_second) ;
         if (len1 != EOF) len += len1 ;
         start_time = start_hour * 3600 + start_minute * 60 + start_second ;

         if ((start_hour >= 24) || (start_minute >= 60) || (start_second >= 60) || (len == 0))
         {
            LoadString(MyInstance, STR_TIMEERROR, strng1, sizeof(strng1));
         }
         else
         {
            sprintf(strng1, "%2.2i:%2.2i:%2.2i", start_hour, start_minute, start_second) ;
            SetDlgItemText(MainWindow, DLG_AT, strng1) ;
            flags[0] = (IsDlgButtonChecked(MainWindow, CB_SU) == BST_CHECKED) ? '1' : '0' ;
            flags[1] = (IsDlgButtonChecked(MainWindow, CB_MO) == BST_CHECKED) ? '1' : '0' ;
            flags[2] = (IsDlgButtonChecked(MainWindow, CB_TU) == BST_CHECKED) ? '1' : '0' ;
            flags[3] = (IsDlgButtonChecked(MainWindow, CB_WE) == BST_CHECKED) ? '1' : '0' ;
            flags[4] = (IsDlgButtonChecked(MainWindow, CB_TH) == BST_CHECKED) ? '1' : '0' ;
            flags[5] = (IsDlgButtonChecked(MainWindow, CB_FR) == BST_CHECKED) ? '1' : '0' ;
            flags[6] = (IsDlgButtonChecked(MainWindow, CB_SA) == BST_CHECKED) ? '1' : '0' ;
            flags[7] = (IsDlgButtonChecked(MainWindow, CB_KEEPOPEN) == BST_CHECKED) ? '1' : '0' ;
            flags[8] = '\0' ;
            put_history(&phistory, HISTORY_SIZE, exefilename, strng1, flags) ;
            init_combobox() ;
            EnableWindow(GetDlgItem(MainWindow, BUT_EXECUTE), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, BUT_START), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, BUT_AT), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, DLG_EXECUTEFILE), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, DLG_AT), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, CB_KEEPOPEN), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, CB_SU), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, CB_MO), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, CB_TU), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, CB_WE), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, CB_TH), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, CB_FR), FALSE) ;
            EnableWindow(GetDlgItem(MainWindow, CB_SA), FALSE) ;
            if (hThread == NULL)
            {
               hThread = CreateThread(
                   NULL,                        // no security attributes
                   0,                           // use default stack size
                   WaitExecute,                 // thread function
                   &dwThrdParam,                // argument to thread function
                   0,                           // use default creation flags
                   &dwThreadId);                // returns the thread identifier
            }

           // Check the return value for success.
            if (hThread == NULL)
               LoadString(MyInstance, STR_THREADSTARTERROR, strng1, sizeof(strng1));
            else
            {
               LoadString(MyInstance, STR_WORKING, strng2, sizeof(strng2));
               sprintf(strng1, strng2, start_hour, start_minute, start_second) ;
            }
         }
         SetDlgItemText(MainWindow, DLG_STATUS, strng1) ;
      break ;

      case BUT_CANCEL:
         abort_thread = TRUE ;
      break ;

      case IDABORT:
      case IDCANCEL:
      case BUT_EXIT:
         GetWindowRect(MainWindow, &rect) ; /* get current dialog coordinates */
         registry_x = rect.left ;
         registry_y = rect.top ;
         registry_width = rect.right - rect.left ;
         registry_height = rect.bottom - rect.top ;
         if (registry_x < 0) registry_x = 0 ;
         if (registry_y < 0) registry_y = 0 ;
         registry_version = PROGRAMVERSION ;    /* version of program which wrote the registry */
         put_registry(&myreg) ;                 /* store the dialog properties in registry */
         EndDialog(MainWindow, TRUE) ;          /* exits the dialog box */
      break ;

      default:
         retcd = 0 ; /* we have not processed this command, so return zero */
      break ;
      }
   break ;

   default:
      retcd = WinLibMessage(msg, wparam, lparam) ; /* WinLib processes some messages */
   break ;
   }

   return(retcd) ;
} /* ExecuteAt */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       init_dialog                                                                                                     |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Initialize dialog elements from history string (time and flags).                                                      |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void init_dialog(_BYTE *strng)
{
_WORD i ;
_BYTE static mystrng[STRINGSIZE] ;
   i = strlen(strng) + 1 ;
   strcpy(mystrng, &strng[i]) ;
   SetDlgItemText(MainWindow, DLG_AT, mystrng) ;
   i = i + strlen(&strng[i]) + 1 ;
   strcpy(mystrng, &strng[i]) ;
   CheckDlgButton(MainWindow, CB_SU,       (mystrng[0] == '1') ? BST_CHECKED : BST_UNCHECKED) ;
   CheckDlgButton(MainWindow, CB_MO,       (mystrng[1] == '1') ? BST_CHECKED : BST_UNCHECKED) ;
   CheckDlgButton(MainWindow, CB_TU,       (mystrng[2] == '1') ? BST_CHECKED : BST_UNCHECKED) ;
   CheckDlgButton(MainWindow, CB_WE,       (mystrng[3] == '1') ? BST_CHECKED : BST_UNCHECKED) ;
   CheckDlgButton(MainWindow, CB_TH,       (mystrng[4] == '1') ? BST_CHECKED : BST_UNCHECKED) ;
   CheckDlgButton(MainWindow, CB_FR,       (mystrng[5] == '1') ? BST_CHECKED : BST_UNCHECKED) ;
   CheckDlgButton(MainWindow, CB_SA,       (mystrng[6] == '1') ? BST_CHECKED : BST_UNCHECKED) ;
   CheckDlgButton(MainWindow, CB_KEEPOPEN, (mystrng[7] == '1') ? BST_CHECKED : BST_UNCHECKED) ;
   return ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       init_combobox                                                                                                   |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Get the command line history and initialize the combo box                                                             |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void init_combobox(void)
{
_WORD i ;
HWND item ;

   item = GetDlgItem(MainWindow, DLG_EXECUTEFILE) ;

// Decode history from registry_history
   get_history(&phistory, HISTORY_SIZE) ;

// Clear combo box
   for (i=0; i<HISTORY_SIZE; i++) SendMessage(item, CB_DELETESTRING, 0, 0);

// Initialize combo box
   for (i=0; i<HISTORY_SIZE; i++)
   {
      if (strlen(phistory[i]) != 0)
         SendMessage(item, CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) phistory[i]);
   }
   SendMessage(item, CB_SETCURSEL, 0, 0) ; // default is the first entry
   init_dialog(phistory[0]) ;
//   SendMessage(item, EM_SETSEL, 0, 0); /* position to top left */
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       get_history                                                                                                     |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Get the command line history from registry_history                                                                    |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void get_history(_BYTE **phistory, _WORD size)
{
_WORD i, j, k, l ;
BOOL done ;
_WORD itemcntr ;

   j = 0 ;
   l = 0 ;
   done = FALSE ;
   for (i=0; ((i<size) && !done); i++)
   {
      done = FALSE ;
      k=0 ;
      itemcntr = 0 ;
      do
      {
         phistory[j][k] = registry_history[l] ;
         if (registry_history[l] == '\0') itemcntr++ ;
         k++ ;
         l++ ;
      }
      while (itemcntr < HISTORY_ITEMS) ;
      phistory[j][k] = '\0' ;
      done = (registry_history[l] == '\0') ;
      j++ ;
   }
   while (j < size) *phistory[j++] = '\0' ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       put_history                                                                                                     |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Put the command line history including the latest command back into the registry_history                              |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void put_history(_BYTE **phistory, _WORD size, _BYTE *newcommand, _BYTE *time, _BYTE *flags)
{
_WORD i, j, k, l, m ;
_BYTE *data ;
_BYTE static cmd[MAX_CMDLINE] ;

// Get fresh copy of history first
   get_history(phistory, size) ;

   j = 0 ;
   m = 0 ;
   for (i=0; i<size+1; i++)
   {
      if (i == 0)
      {
         k = 0 ; l = 0 ;
         do {cmd[k++] = newcommand[l] ;} while (newcommand[l++] != '\0') ;
         l = 0 ;
         do {cmd[k++] = time[l] ;} while (time[l++] != '\0') ;
         l = 0 ;
         do {cmd[k++] = flags[l] ;} while (flags[l++] != '\0') ;
         cmd[k] = '\0' ;
         data = cmd ;
      }
      else
      {
         data = phistory[i-1] ;
         if (strcmp(data, newcommand) == 0) data = NULL ; // ignore if identical to current command
      }
      if ((data != NULL) && (j < size))
      {
         k = 0 ;
         do
         {
            registry_history[m] = data[k] ;
            m++ ; k++ ;
         } while ((data[k-1] != '\0') || (data[k] != '\0')) ;
      }
   }
   registry_history[m] = '\0' ;
// Get fresh copy of history once again
   get_history(phistory, size) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       DWORD WINAPI WaitExecute(LPVOID lpParam)                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Wait for time and execute command                                                                                     |
| The user can abort this thread by setting the global variable abort_thread to TRUE.                                   |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
DWORD WINAPI WaitExecute(LPVOID lpParam)
{
SYSTEMTIME systime ;
PROCESS_INFORMATION pi;
STARTUPINFO si;
_WORD current_time ;
_WORD maxtime ;
_BOOL ready ;
_BOOL day_ok ;
_WORD dayofweek ;

   maxtime = start_time + TRIGGER_MARGIN ;
   while (!abort_thread)
   {
      GetLocalTime(&systime) ;
      dayofweek = systime.wDayOfWeek ;
      current_time = systime.wHour * 3600 + systime.wMinute * 60 + systime.wSecond ;
// Handle wrapping at midnight.
      if (maxtime >= (current_time + 24*3600))
      { // start time was before midnight, but midnight has already passed
         current_time += 24*3600 ;
         if (dayofweek == 0) dayofweek = 6 ;
         else dayofweek -= 1 ; // correct the weekday
      }
      day_ok = (((dayofweek == 0) && (flags[0] == '1'))
             || ((dayofweek == 1) && (flags[1] == '1'))
             || ((dayofweek == 2) && (flags[2] == '1'))
             || ((dayofweek == 3) && (flags[3] == '1'))
             || ((dayofweek == 4) && (flags[4] == '1'))
             || ((dayofweek == 5) && (flags[5] == '1'))
             || ((dayofweek == 6) && (flags[6] == '1')) ) ;
      if ((current_time < start_time) || (current_time > maxtime))
         ready = TRUE ;
      else
      {
         if (ready && day_ok)
         {
            ready = FALSE ;
            si.cb = sizeof(si) ;
            si.lpDesktop = NULL ;
            si.lpTitle = NULL ;
            si.dwFlags = STARTF_USESHOWWINDOW ;
            si.wShowWindow = SW_MINIMIZE ;
            si.cbReserved2 = 0 ;
            si.lpReserved2 = NULL ;
            CreateProcess(
                          NULL,                    // pointer to name of executable module
                          command ,                // pointer to command line string
                          NULL,                    // pointer to process security attributes
                          NULL,                    // pointer to thread security attributes
                          TRUE,                    // handle inheritance flag
                          NORMAL_PRIORITY_CLASS,   // creation flags
                          NULL,                    // pointer to new environment block
                          NULL,                    // pointer to current directory name
                          &si,                     // pointer to STARTUPINFO
                          &pi);                    // pointer to PROCESS_INFORMATION

// No need to wait (???)
//            WaitForSingleObject(pi.hProcess,INFINITE) ;
         }
      }
      Sleep(SLEEPTIME) ;
   }

// Notify main window that the thread is finished
    SendMessage(MainWindow, WM_USER, USER_THREAD_ENDED, 0) ;

    return 0;
}

