/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|          Program Name: Fix Registry                                                                                   |
|                Author: Michael Krmer                                                                                 |
|                        Michael@dk8pp.de               (private)                                                       |
|                        Michael.Kraemer@eu.necel.com   (business)                                                      |
|                  Date: 30.Jun-2008                                                                                    |
|              Language: Open Watcom C 1.7                                                                              |
|               Version: 0.1                                                                                            |
|        Program status: still testing                                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Some registry settings do not persist. After a while, they are reset to some default values.                |
|           This tiny program is meant to set them to the preferred values. It can be executed manually upon            |
|           necessity or automatically e.g. by putting it into the Autostart folder. Note that some settings            |
|           will only become effective after the next startup and so sometimes the unwanted behaviour applies           |
|           during a session. Something like an autostop folder would be required to fix that...                        |
|                                                                                                                       |
|           Set AutoAdminLogon in registry (aal=0:off, aal=1: on)                                                       |
|           (HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\WinLogon\AutoAdminLogon=1)                 |
|                                                                                                                       |
|           Set/Clear Autostart                                                                                         |
|           HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Policies\Explorer\NoDriveTypeAutoRun            |
|              Bit 0: UNKNOWN                                                                                           |
|              Bit 1: NO_ROOT                                                                                           |
|              Bit 2: REMOVABLE (asrem=0: autostart off, asrem=1: autostart on)                                         |
|              Bit 3: FIXED (asfixed=0: autostart off, asfixed=1: autostart on)                                         |
|              Bit 4: REMOTE                                                                                            |
|              Bit 5: CDROM (ascd=0: autostart off, ascd=1: autostart on)                                               |
|              Bit 6: RAMDISK                                                                                           |
|              Bit 7: FUTURE                                                                                            |
|              Bit=0: Autostart enabled, Bit=1: Autostart disabled                                                      |
|                                                                                                                       |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include <winreg.h>
#include "WinLib.h"

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define STRINGSIZE 256          /* default size for strings */
#define PARMSIZE 20             /* maximum size of a string or positional command line parameter */
#define PROGRAMNAME "Fix Registry"
#define PURPOSE "This program fixes registry settings which do not persist\r\r\n"
#define PROGRAMVERSION 10       /* will be displayed as n.nn */

#define DR_REMOVABLE 0x04
#define DR_FIXED     0x08
#define DR_CDROM     0x20

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
/* the following global variables are required to satisfy winlib. They are not used otherwise,
   because this is a character executable */
HINSTANCE       MyInstance ;
HWND            MainWindow ;
_BYTE strng1[STRINGSIZE] ;
_BYTE strng2[STRINGSIZE] ;
_BOOL disablereg = FALSE ;      /* disable use of the registry */
_BYTE purpose[] = PURPOSE ;

/*
 Command line parameter definitions
*/
_BYTE posparm1[PARMSIZE+1] ;    /* positional parameter 1 */
_BYTE posparm2[PARMSIZE+1] ;    /* positional parameter 2 */
_BYTE strng[PARMSIZE+1] ;       /* string parameter strng */

CLO clo[] = {{"aal", PARM_INT, NULL, 0, FALSE, "AutoAdminLogon (aal={on,off})"},
             {"asrem", PARM_INT, NULL, 0, FALSE, "Autostart for removable drives (asrem={on,off})"},
             {"asfixed", PARM_INT, NULL, 0, FALSE, "Autostart for fixed drives (asfixed={on,off})"},
             {"ascd", PARM_INT, NULL, 0, FALSE, "Autostart for CD-ROM drives (ascd={on,off})"},
             {"h", PARM_SIMPLE, NULL, 0, FALSE, "Print program information"}
             } ;
_WORD clo_size = sizeof(clo)/sizeof(clo[0]) ;

/* These are indexes into the CLO table above. Keep them in sync */
#define PARM_AAL 0
#define PARM_ASREM 1
#define PARM_ASFIXED 2
#define PARM_ASCD 3
#define PARM_HELP 4

_BYTE errorparm[MAXPARMLENGTH] ;


/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       WinMain                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| MAIN program.                                                                                                         |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int WINAPI WinMain(HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline, int cmdshow)
{
HKEY keyhandle ;
LONG retv ;
BYTE *val ;
_LONG valdw ;
_LONG csize ;
   errorparm[0] = '\0' ;
   getparms(cmdline, errorparm) ;
   if (strlen(errorparm) != 0) printf("Invalid command line parameter(s) \"%s\"", errorparm) ;

   if (clo[PARM_HELP].found)
   {
      sprintf(strng1, "Version %u.%2.2u", PROGRAMVERSION/100, PROGRAMVERSION%100) ;
      PrintProgramInfo(0, PROGRAMNAME, strng1, __DATE__, __TIME__) ;
   }

   if (clo[PARM_AAL].found)
   {
      if ((clo[PARM_AAL].value) == 0) val = "0" ;
         else val = "1" ;
// Open key
      retv = RegOpenKeyEx(HKEY_LOCAL_MACHINE,                              // handle of an open key
             "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Winlogon",  // address of subkey name
             0,                            // reserved
             KEY_ALL_ACCESS,               // desired security access
             &keyhandle                    // address of buffer for opened handle
         ) ;
      if (retv == ERROR_SUCCESS)
      {// write data
         retv = RegSetValueEx(
                keyhandle,                 // handle of key to set value for
                "AutoAdminLogon",          // name to set
                0,                         // reserved
                REG_SZ,                    // flag for value type
                val,                       // data
                2                          // size of value data
                ) ;
         RegCloseKey(keyhandle) ;
      }
   }

   if ((clo[PARM_ASREM].found) || (clo[PARM_ASFIXED].found) || (clo[PARM_ASCD].found))
   {
// Open key
      retv = RegOpenKeyEx(HKEY_CURRENT_USER,                               // handle of an open key
             "Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer", // address of subkey name
             0,                            // reserved
             KEY_ALL_ACCESS,               // desired security access
             &keyhandle                    // address of buffer for opened handle
         ) ;
      if (retv == ERROR_SUCCESS)
      {

/* read current value */
         csize = 4 ;
         retv = RegQueryValueEx(
                keyhandle,                        // handle of key to query
                "NoDriveTypeAutoRun",             // address of name of value to query
                0,                                // reserved
                NULL,                             // address of buffer for value type
                (LPBYTE)&valdw,                   // address of data buffer
                &csize                            // address of data buffer size
                ) ;
         if (retv == ERROR_SUCCESS)
         {
            if ((clo[PARM_ASREM].found) && (clo[PARM_ASREM].value == 0)) valdw = valdw | DR_REMOVABLE ;
            if ((clo[PARM_ASREM].found) && (clo[PARM_ASREM].value == 1)) valdw = valdw & ~DR_REMOVABLE ;
            if ((clo[PARM_ASFIXED].found) && (clo[PARM_ASFIXED].value == 0)) valdw = valdw | DR_FIXED ;
            if ((clo[PARM_ASFIXED].found) && (clo[PARM_ASFIXED].value == 1)) valdw = valdw & ~DR_FIXED ;
            if ((clo[PARM_ASCD].found) && (clo[PARM_ASCD].value == 0)) valdw = valdw | DR_CDROM ;
            if ((clo[PARM_ASCD].found) && (clo[PARM_ASCD].value == 1)) valdw = valdw & ~DR_CDROM ;
            {// write data
               retv = RegSetValueEx(
                      keyhandle,                 // handle of key to set value for
                      "NoDriveTypeAutoRun",      // name to set
                      0,                         // reserved
                      REG_DWORD,                 // flag for value type
                      (CONST BYTE *)&valdw,      // data
                      4                          // size of value data
                      ) ;
               RegCloseKey(keyhandle) ;
            }
         }
      }
   }

   return(0);
}

