/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|          Program Name: Mytype.h                                                                                       |
|                Author: Michael Krmer                                                                                 |
|                        Michael@dk8pp.de               (private)                                                       |
|                  Date: 20.Feb.2008                                                                                    |
|              Language: C                                                                                              |
|               Version: 2.0                                                                                            |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Include file to define some common data types.                                                              |
|                                                                                                                       |
|  History:                                                                                                             |
|  This file was originally made for DOS and embedded applications, but it turned out to be incompatible with           |
|  the Windows development environment, mainly because of the WORD data types. By definition, a WORD should be          |
|  architecture dependent, i.e. 16 bits under DOS and any 16-bit processor, 32 bits under Windows and any other         |
|  32-bit processor. For Windows, a WORD is defined as short int, i.e. 16 bits, surely for historical reasons.          |
|                                                                                                                       |
|  Now I'm used to my data types and I don't want to change my old programs. In order to avoid conflicts with           |
|  Windows, I decided to append a leading underscore to my own definitions. For those who don't like this, it is        |
|  easy to change these definitions by a global change.                                                                 |
|                                                                                                                       |
|  The following definitions apply:                                                                                     |
|                                                                                                                       |
|   |====================|============|==========|                                                                      |
|   |   Data type        | Mytype.h   | Windows  |                                                                      |
|   |--------------------|------------|----------|                                                                      |
|   | signed char        | _SCHAR     | SCHAR    |                                                                      |
|   | char               |            | CHAR     |                                                                      |
|   | unsigned char      | _CHAR      | BYTE     |                                                                      |
|   | signed char        | _SBYTE     |          |                                                                      |
|   | unsigned char      | _BYTE      | BYTE     |                                                                      |
|   | signed int         | _SWORD     |          |                                                                      |
|   | unsigned int       | _WORD      |          |                                                                      |
|   | signed long        | _SLONG     | SLONG    |                                                                      |
|   | long               |            | LONG     |                                                                      |
|   | signed long long   | _SLONGLONG |          |                                                                      |
|   | unsigned long      | _LONG      |          |                                                                      |
|   | unsigned long long | _LONGLONG  |          |                                                                      |
|   | signed short int   | _SSHORT    | SSHORT   |                                                                      |
|   | short              |            | SHORT    |                                                                      |
|   | unsigned short int | _SHORT     | WORD     |                                                                      |
|   | short              |            | SWORD    |                                                                      |
|   | int                | _BOOL      | BOOL     |                                                                      |
|   | float              | _FLOAT     | FLOAT    |                                                                      |
|   | double             | _DOUBLE    | DOUBLE   |                                                                      |
|   | double             |            | LDOUBLE  |                                                                      |
|   | long double        | _LDOUBLE   |          |                                                                      |
|   |====================|============|==========|                                                                      |
|                                                                                                                       |
|  The major difficulty is probably the WORD datatype, which is 16-bit unsigned, while _WORD is architecture            |
|  dependent (on purpose) and is 32-bit under Windows. If an object must be 32-bit, its data type should be _LONG       |
|  or _SLONG.                                                                                                           |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef MYTYPE

typedef signed char             _SCHAR ;
typedef unsigned char           _CHAR ;
typedef signed char             _SBYTE ;
typedef unsigned char           _BYTE ;
typedef signed int              _SWORD ;
typedef unsigned int            _WORD ;
typedef signed long             _SLONG ;
typedef signed long long        _SLONGLONG ;
typedef unsigned long           _LONG ;
typedef unsigned long long      _LONGLONG ;
typedef signed short int        _SSHORT ;
typedef unsigned short int      _SHORT ;
typedef int                     _BOOL ;
typedef float                   _FLOAT ;
typedef double                  _DOUBLE ;
typedef long double             _LDOUBLE ;

#define REGISTER        register unsigned int

#ifndef TRUE
#define TRUE  1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#define MYTYPE 1
#endif
