/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|          Program Name: WinLib.h                                                                                       |
|                Author: Michael Krmer                                                                                 |
|                        Michael@dk8pp.de               (private)                                                       |
|                  Date: 30-Nov-2007                                                                                    |
|              Language: Open Watcom C 1.7                                                                              |
|               Version: 1.0                                                                                            |
|        Program status: First public release                                                                           |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Library with some often used functions for Windows programming                                              |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#ifndef WINLIB
#include "Mytype.h"

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define WINLIB 1
#define RESIZE_X 0x1            /* resize control in x direction */
#define RESIZE_Y 0x2            /* resize control in y direction */
#define MOVE_X   0x4            /* move control in x direction */
#define MOVE_Y   0x8            /* move control in y direction */
#define MY_REG_KEY_ROOT "Software\\Michael Krmer Freeware\\" // Root for registry keys under HKEY_CURRENT_USER
#define PARM_SIMPLE 1000        /* simple parameter, no argument (eg. -a, -rc etc.) */
#define PARM_INT 1001           /* integer parameter (eg. -q=-25) */
#define PARM_STRING 1002        /* string parameter (eg. -f=image.jpg, -f="c:\Program Files\image.jpg") */
#define PARM_POS 1003           /* positional parameter (eg. image.jpg, "c:\Program Files\image.jpg") */
#define MAXPARMLENGTH 128       /* maximum length of a command line parameter */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/
typedef struct {
        _WORD control ;
        _WORD left ;
        _WORD top ;
        _WORD right ;
        _WORD bottom ;
        _WORD flags ;
                } COORDS ;

typedef struct {
        _BYTE *valuename ;
        _WORD valuetype ;
        void *value ;
        _WORD valuesize ;
                } REGVALUE ;

typedef struct {
        HKEY hkey ;            /* handle of open key */
        _BYTE *regkeyname ;    /* pointer to name of registry key */
        REGVALUE *regvalues ;
        _WORD size ;
                } MYREG ;

typedef struct { /* command line options */
        _BYTE *parmname ; /* name of parameter without leading "-" or "/" */
        _WORD parmtype ;  /* type of this parameter */
         void *dest ;     /* pointer to destination for a string parameter */
        _SWORD value ;    /* value of this parameter or maximum stringlength in case of a string */
        _BOOL found ;     /* parameter was found */
        _BYTE *help ;     /* help text for this parameter */
                } CLO ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void display_info(_BYTE *msg) ;
void ShowError(int dialog) ;
void resize_controls(COORDS *coordinates, _WORD size) ;
void get_coords(COORDS *coordinates, _WORD size) ;
int compare_hup(const void *op1, const void *op2) ;
int compare_hdown(const void *op1, const void *op2) ;
int compare_vup(const void *op1, const void *op2) ;
int compare_vdown(const void *op1, const void *op2) ;
WORD WinLibMessage(unsigned msg, WPARAM wparam, LPARAM lparam) ;
void SaveCoordinates(void) ;
_BOOL get_registry(MYREG *myreg) ;
void put_registry(MYREG *myreg) ;
void clear_registry(MYREG *myreg) ;
void PrintProgramInfo(_WORD dlgitem, _BYTE *programname, _BYTE *programversion, _BYTE *date, _BYTE *time) ;
void getparms(LPSTR cmdline, _BYTE *errorparm) ;
void set_process_prio(_SWORD prio) ;
void set_thread_prio(HANDLE hthread, _SWORD prio) ;

#endif

