/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|          Program Name: WinLib                                                                                         |
|                Author: Michael Krmer                                                                                 |
|                        Michael@dk8pp.de               (private)                                                       |
|                        Michael.Kraemer@eu.necel.com   (business)                                                      |
|                  Date: 30-Nov-2007                                                                                    |
|              Language: Open Watcom C 1.7                                                                              |
|               Version: 0.1                                                                                            |
|        Program status: still testing                                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Library with some often used functions for Windows programming                                              |
|                                                                                                                       |
|           Define WIN_CHARACTERMODE for character mode programs                                                        |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include "WinDef.h"
#include "mytype.h"
#include "WinLib.h"

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define STRINGSIZE 256          /* default size for strings */
#define SNAP 20                 /* SNAP when moving window */

#define EMAILADDR "Michael@dk8pp.de"
#define WEBSITE "http://downloads.dk8pp.de/"
#ifdef WIN_CHARACTERMODE
#define PROGRAMMER "Michael Krmer"
#define NEWLINE "\n"
#else
#define PROGRAMMER "Michael Krmer"
#define NEWLINE "\r\r\n"
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
extern HWND      MainWindow ;
extern HINSTANCE MyInstance ;
HWND             HelpWindow ;
extern CLO clo[] ;
extern _WORD clo_size ;
extern _BOOL disablereg ;
extern _BYTE purpose[] ;
_BYTE local_strng1[STRINGSIZE] ;
int process_prio[] = {IDLE_PRIORITY_CLASS, NORMAL_PRIORITY_CLASS, HIGH_PRIORITY_CLASS, REALTIME_PRIORITY_CLASS} ;
int thread_prio[] = {THREAD_PRIORITY_IDLE, THREAD_PRIORITY_LOWEST, THREAD_PRIORITY_BELOW_NORMAL,
                     THREAD_PRIORITY_NORMAL, THREAD_PRIORITY_ABOVE_NORMAL, THREAD_PRIORITY_HIGHEST,
                     THREAD_PRIORITY_TIME_CRITICAL} ;
_WORD helpdlgitem ;
_WORD min_size_x ;      /* Minimum horizontal dialog size */
_WORD min_size_y ;      /* Minimum vertical dialog size */
#ifndef WIN_CHARACTERMODE
/* The following externs are only defined in windowed mode */
extern COORDS coordinates[] ;
extern COORDS *sorted_coord[] ;
extern WORD coord_size ;
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/
BOOL CALLBACK DlgHelp(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam) ;
_BOOL wl_get_token(_BYTE *in, _BYTE *out) ;
_WORD str2len(_BYTE *str) ;
_BYTE *GetHelp(_WORD clpnr, _BYTE *strng) ;

#ifndef WIN_CHARACTERMODE
/* These functions make sense only in windowed mode */
/*
+-----------------------------------------------------------------------------------------------------------------------+
|       WinLibMessage                                                                                                   |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| This function processes some of the dialog messages.                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
WORD WinLibMessage(unsigned msg, WPARAM wparam, LPARAM lparam)
{
WORD retcd ;
WPARAM fwSide ;
LPRECT lprc ;
RECT rect ;
_SWORD xPos ;
_SWORD yPos ;
_SWORD xSize ;
_SWORD ySize ;
_BOOL movewin ;

   retcd = 0 ; /* return 0 if we don't process the message */

   switch(msg)
   {
   case WM_SIZE:
      retcd = 0 ;
      resize_controls(&coordinates[0], coord_size) ;
   break ;

   case WM_SIZING:
      retcd = TRUE ;
      fwSide = wparam;         // edge of window being sized
      lprc = (LPRECT) lparam;  // screen coordinates of drag rectangle
      if ((fwSide == WMSZ_RIGHT) || (fwSide == WMSZ_BOTTOMRIGHT) || (fwSide == WMSZ_TOPRIGHT))
         if ((lprc->right-lprc->left) < min_size_x) lprc->right = lprc->left + min_size_x ;
      if ((fwSide == WMSZ_LEFT) || (fwSide == WMSZ_BOTTOMLEFT) || (fwSide == WMSZ_TOPLEFT))
         if ((lprc->right-lprc->left) < min_size_x) lprc->left = lprc->right - min_size_x ;
      if ((fwSide == WMSZ_BOTTOM) || (fwSide == WMSZ_BOTTOMRIGHT) || (fwSide == WMSZ_BOTTOMLEFT))
         if ((lprc->bottom-lprc->top) < min_size_y) lprc->bottom = lprc->top + min_size_y ;
      if ((fwSide == WMSZ_TOP) || (fwSide == WMSZ_TOPRIGHT) || (fwSide == WMSZ_TOPLEFT))
         if ((lprc->bottom-lprc->top) < min_size_y) lprc->top = lprc->bottom - min_size_y ;
   break ;

   case WM_MOVE:
      retcd = TRUE ;
      movewin = FALSE ;
      GetWindowRect(MainWindow, &rect) ;
      xPos = rect.left ;    // horizontal position
      yPos = rect.top ;    // vertical position
      if (abs(xPos) < SNAP) {xPos = 0 ; movewin = TRUE ;}
      if (abs(yPos) < SNAP) {yPos = 0 ; movewin = TRUE ;}
      xSize = GetSystemMetrics(SM_CXFULLSCREEN) ;
      ySize = GetSystemMetrics(SM_CYFULLSCREEN) ;
      if (abs(xSize - rect.right) < SNAP) {xPos = xSize - (rect.right - rect.left) ; movewin = TRUE ;}
      if (abs(ySize - rect.bottom) < SNAP) {yPos = ySize - (rect.bottom - rect.top) ; movewin = TRUE ;}
      if (movewin) MoveWindow(
                   MainWindow,            // handle of window
                   xPos,            // horizontal position
                   yPos,            // vertical position
                   rect.right - rect.left,        // width
                   rect.bottom - rect.top,       // height
                   TRUE);                 // repaint flag
   break ;

   }
   return(retcd) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       SaveCoordinates                                                                                                 |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Save the coordinates of the dialog and of the controls.                                                               |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void SaveCoordinates(void)
{
RECT rect ;
// save original coordinates of dialog
   GetWindowRect(MainWindow, &rect) ;
   coordinates[0].top = rect.top ;
   coordinates[0].right = rect.right ;
   coordinates[0].bottom = rect.bottom ;
   coordinates[0].left = rect.left ;
   min_size_x = rect.right - rect.left ; /* the original dialog size is the minimum size */
   min_size_y = rect.bottom - rect.top ; /* the original dialog size is the minimum size */

// save original coordinates of controls
   get_coords(&coordinates[0], coord_size) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       resize_controls                                                                                                 |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| This function is called as a response to the WM_SIZE message. It resizes the controls of a dialog.                    |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void resize_controls(COORDS *coordinates, _WORD size)
{
RECT rect ;
_WORD i, hsize, vsize, hsize_org, vsize_org, addx, addy, movx, movy ;
_WORD static hsize_prev = 0 ;
_WORD static vsize_prev = 0 ;
// get current size of dialog
   GetWindowRect(MainWindow, &rect) ;
   hsize = rect.right - rect.left ;
   vsize = rect.bottom - rect.top ;
   hsize_org = coordinates[0].right-coordinates[0].left ;
   vsize_org = coordinates[0].bottom-coordinates[0].top ;

   if (hsize_prev == 0) hsize_prev = hsize_org ;
   if (vsize_prev == 0) vsize_prev = vsize_org ;

/* horizontal sizing */
   if (hsize_prev != hsize)
   {
/* Controls must be sorted so that leftmost controls are moved first, when dialog gets smaller and rightmost
   controls are moved first when dialog gets larger */
      for (i=1; i<size; i++) sorted_coord[i-1] = &coordinates[i] ;
      if (hsize < hsize_prev)
      { /* dialog gets smaller */
         qsort(&sorted_coord[0], i-1, sizeof(COORDS *), compare_hup) ;
      }
      else
      { /* dialog gets larger */
         qsort(&sorted_coord[0], i-1, sizeof(COORDS *), compare_hdown) ;
      }

      for (i=1; i<size; i++)
      {
         if (((sorted_coord[i-1]->flags)&RESIZE_X)!=0) addx = hsize-hsize_org ; else addx = 0 ;
         if (((sorted_coord[i-1]->flags)&RESIZE_Y)!=0) addy = vsize_prev-vsize_org ; else addy = 0 ;
         if (((sorted_coord[i-1]->flags)&MOVE_X)!=0) movx = hsize-hsize_org ; else movx = 0 ;
         if (((sorted_coord[i-1]->flags)&MOVE_Y)!=0) movy = vsize_prev-vsize_org ; else movy = 0 ;
         MoveWindow(
         GetDlgItem(MainWindow, sorted_coord[i-1]->control),    // handle of window
         sorted_coord[i-1]->left+movx,                          // horizontal position
         sorted_coord[i-1]->top+movy,                           // vertical position
         sorted_coord[i-1]->right-sorted_coord[i-1]->left+addx, // width
         sorted_coord[i-1]->bottom-sorted_coord[i-1]->top+addy, // height
         TRUE);                                                 // repaint flag
      }
   }
   hsize_prev = hsize ;

/* vertical sizing */
   if (vsize_prev != vsize)
   {
/* Controls must be sorted so that topmost controls are moved first, when dialog gets smaller and bottommost
   controls are moved first when dialog gets larger */
      for (i=1; i<size; i++) sorted_coord[i-1] = &coordinates[i] ;
      if (vsize < vsize_prev)
      { /* dialog gets smaller */
         qsort(&sorted_coord[0], i-1, sizeof(COORDS *), compare_vup) ;
      }
      else
      { /* dialog gets larger */
         qsort(&sorted_coord[0], i-1, sizeof(COORDS *), compare_vdown) ;
      }
      for (i=1; i<size; i++)
      {
         if (((sorted_coord[i-1]->flags)&RESIZE_X)!=0) addx = hsize_prev-hsize_org ; else addx = 0 ;
         if (((sorted_coord[i-1]->flags)&RESIZE_Y)!=0) addy = vsize-vsize_org ; else addy = 0 ;
         if (((sorted_coord[i-1]->flags)&MOVE_X)!=0) movx = hsize_prev-hsize_org ; else movx = 0 ;
         if (((sorted_coord[i-1]->flags)&MOVE_Y)!=0) movy = vsize-vsize_org ; else movy = 0 ;
         MoveWindow(
         GetDlgItem(MainWindow, sorted_coord[i-1]->control),    // handle of window
         sorted_coord[i-1]->left+movx,                          // horizontal position
         sorted_coord[i-1]->top+movy,                           // vertical position
         sorted_coord[i-1]->right-sorted_coord[i-1]->left+addx, // width
         sorted_coord[i-1]->bottom-sorted_coord[i-1]->top+addy, // height
         TRUE);                                                 // repaint flag
      }
   }
   vsize_prev = vsize ;

}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       int compare_hup(const void *op1, const void *op2 )                                                              |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Compare for qsort                                                                                                     |
| For horizontal moves to the left                                                                                      |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int compare_hup(const void *op1, const void *op2)
{
COORDS *p1, *p2 ;
   p1 = * (COORDS **) op1 ;
   p2 = * (COORDS **) op2 ;
   return (p1->left - p2->left) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       int compare_hdown(const void *op1, const void *op2 )                                                            |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Compare for qsort                                                                                                     |
| For horizontal moves to the right                                                                                     |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int compare_hdown(const void *op1, const void *op2)
{
COORDS *p1, *p2 ;
   p1 = * (COORDS **) op1 ;
   p2 = * (COORDS **) op2 ;
   return (p2->left - p1->left) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       int compare_vup(const void *op1, const void *op2 )                                                              |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Compare for qsort                                                                                                     |
| For vertical moves to the top                                                                                         |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int compare_vup(const void *op1, const void *op2)
{
COORDS *p1, *p2 ;
   p1 = * (COORDS **) op1 ;
   p2 = * (COORDS **) op2 ;
   return (p1->top - p2->top) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       int compare_vdown(const void *op1, const void *op2 )                                                            |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Compare for qsort                                                                                                     |
| For vertical moves to the bottom                                                                                      |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int compare_vdown(const void *op1, const void *op2)
{
COORDS *p1, *p2 ;
   p1 = * (COORDS **) op1 ;
   p2 = * (COORDS **) op2 ;
   return (p2->top - p1->top) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       get_coords                                                                                                      |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Get the coordinates of the controls, which are defined in the coordinates structure.                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void get_coords(COORDS *coordinates, _WORD size)
{
RECT rect ;
POINT pntlo ;
POINT pntru ;
_WORD i ;
   for (i=1; i<size; i++) /* skip first entry; reserved for main dialog */
   {
      GetWindowRect(GetDlgItem(MainWindow, coordinates[i].control), &rect) ;
      pntlo.x = rect.left ;
      pntlo.y = rect.top ;
      pntru.x = rect.right ;
      pntru.y = rect.bottom ;
      ScreenToClient(MainWindow, &pntlo) ;
      ScreenToClient(MainWindow, &pntru) ;
      coordinates[i].left = pntlo.x ;
      coordinates[i].top = pntlo.y ;
      coordinates[i].right = pntru.x ;
      coordinates[i].bottom = pntru.y ;
   }
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       DlgHelp                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Help Dialog                                                                                                           |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
BOOL CALLBACK DlgHelp(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam)
{
WORD retcd ; /* return nonzero, if the message is processed here and zero, if it is not */
HWND msg_hwnd ;
_WORD i ;

   retcd = 0 ; /* return 0 if we don't process the message */
   switch(msg)
   {
   case WM_INITDIALOG:
      retcd = 1 ;
      HelpWindow = window_handle ;
      msg_hwnd = GetDlgItem(HelpWindow, helpdlgitem) ;
      SendMessage(msg_hwnd, EM_SETSEL, 0, -1); /* clear window */
      SendMessage(msg_hwnd, EM_REPLACESEL, (BOOL) FALSE, (WPARAM) local_strng1) ;
      SendMessage(msg_hwnd, EM_REPLACESEL, (BOOL) FALSE, (WPARAM) purpose) ;
      SendMessage(msg_hwnd, EM_REPLACESEL, (BOOL) FALSE, (WPARAM) "The following command line parameters are supported:" NEWLINE) ;
      for (i=0; i<clo_size; i++) SendMessage(msg_hwnd, EM_REPLACESEL, (BOOL) FALSE, (WPARAM) GetHelp(i, local_strng1)) ;
   break ;

   case WM_COMMAND:
      retcd = 1 ;
      switch(LOWORD(wparam))
      {
      case IDOK:
      case IDABORT:
      case IDCANCEL:
         EndDialog(HelpWindow, TRUE);      /* exits the dialog box */
      break ;

      default:
         retcd = 0 ; /* we have not processed this command, so return zero */
      break ;
      }
   break ;
   }

   return(retcd) ;
} /* DlgHelp */
#endif

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       GetHelp                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Get Help for a specific command line parameter                                                                        |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_BYTE *GetHelp(_WORD clpnr, _BYTE *strng)
{
_BYTE *prefix ;
_BYTE *parmname ;
_BYTE *parmval ;
_BYTE parm[16] ;
_WORD pos ;

   pos = 1 ;
   switch (clo[clpnr].parmtype)
   {
      case PARM_SIMPLE: prefix = "-" ; parmname = clo[clpnr].parmname ; parmval = "" ; break ;
      case PARM_INT: prefix = "-" ; parmname = clo[clpnr].parmname ; parmval = "=n" ; break ;
      case PARM_STRING: prefix = "-" ; parmname = clo[clpnr].parmname ; parmval = "=\"string\"" ; break ;
      case PARM_POS: prefix = "" ; sprintf(parm, "pos%u", pos) ; parmname = parm ; pos++; parmval = "" ; break ;
      default: prefix = "" ; parmname = "" ; parmval = "" ; break ;
   }
   sprintf(strng, "%s%s%s: %s" NEWLINE, prefix, parmname, parmval, clo[clpnr].help) ;

   return(strng) ;
} /* GetHelp */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       PrintProgramInfo                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Open a message box and print some information about the program                                                       |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void PrintProgramInfo(_WORD dlgitem, _BYTE *programname, _BYTE *programversion, _BYTE *date, _BYTE *time)
{
#ifdef WIN_CHARACTERMODE
_WORD i ;
#endif
   helpdlgitem = dlgitem ;
   sprintf(local_strng1,"%s - %s " NEWLINE
                 "Written by: " PROGRAMMER ", %s, %s" NEWLINE
                 "for updates check: " WEBSITE NEWLINE
                 "or email to: " EMAILADDR NEWLINE NEWLINE,
                 programname, programversion, date, time) ;
#ifdef WIN_CHARACTERMODE
   printf("%s", local_strng1) ;
   printf("The following command line parameters are supported:\n") ;
   for (i=0; i<clo_size; i++) printf("%s", GetHelp(i, local_strng1)) ;
#else
   DialogBox(MyInstance, "DLG_HELP", MainWindow, (DLGPROC)DlgHelp) ;
#endif
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       wl_get_token                                                                                                    |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Get the next token from the input string                                                                              |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_BOOL wl_get_token(_BYTE *in, _BYTE *out)
{
_WORD i, j ;
_BOOL terminate, hyphen, rc ;
   i = 0 ;
   j = 0 ;
   terminate = FALSE ;
   hyphen = FALSE ;
   rc = FALSE ;
   while (!terminate)
   {
      out[j] = in[i] ;
      if (in[i] == '\0') terminate = TRUE ;
      if ((in[i] == ' ') && !hyphen)
      {
         out[j] = '\0' ;
         terminate = TRUE ;
      }
      if ((in[i] == '"') && hyphen)
      {
         out[j+1] = '\0' ;
         terminate = TRUE ;
      }
      hyphen = hyphen || (in[i] == '"') ;
      i++ ;
      j++ ;
   }
   if (out[0] != '\0') rc = TRUE ;
   return(rc) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       getparms                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Get the command line parameters                                                                                       |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void getparms(LPSTR cmdline, _BYTE *errorparm)
{
_BOOL cmd_done, found_eq ;
_BYTE static token[MAXPARMLENGTH] ;
_BYTE *parmvalue ;
_WORD i, j, k ;
_SWORD intval ;
_BOOL found ;
   cmd_done = FALSE ;
   errorparm[0] = '\0' ;
/* check parameters if cmdline points to a string */
   if ((cmdline != NULL) && (cmdline[0] != '\0'))
   {
      i = 0 ;
      while (!cmd_done)
      {
         while (cmdline[i] == ' ') i++ ; /* skip any possibly leading blanks */
         if (cmdline[i] == '\0') cmd_done = TRUE ;
         else
         {
            if ((cmdline[i] == '-') || (cmdline[i] == '/'))
            {/* this is a parameter */
               i++ ;
               if ((cmdline[i] == ' ') || (cmdline[i] == '\0')) /* we found a "-" without a parameter */
               {
                  token[0] = '\0' ;
               }
               else
               {
                  if (wl_get_token(&cmdline[i], &token) == FALSE) cmd_done = TRUE ;
               }
               if (!cmd_done)
               { /* we isolated a parameter. Check if it is an assignment. */
                  found_eq = FALSE ;
                  k = 0 ;
                  while ((token[k] != '\0') && (token[k] != '=')) k++ ;
                  if (token[k] == '=')
                  {
                     found_eq = TRUE ;
                     token[k] = '\0' ; /* terminator for parameter name */
                     parmvalue = &token[k+1] ;
                  }
                  /* Now we search the parameter name in the list. */
                  found = FALSE ;
                  for (j=0; (j<clo_size) && (!found); j++)
                  {
                     if ((clo[j].parmtype == PARM_SIMPLE)
                      || (clo[j].parmtype == PARM_INT)
                      || (clo[j].parmtype == PARM_STRING))
                     found = strcmp(token, clo[j].parmname) == 0 ;
                  }
                  if (found_eq) token[k] = '=' ; /* restore = */
                  if (found)
                  {
                     j-- ; /* fix increment from previous for loop */
                     if (clo[j].parmtype == PARM_SIMPLE) clo[j].found = TRUE ;
                     if (clo[j].parmtype == PARM_INT)
                     { /* convert parmvalue to int */
                        if (clo[j].found == TRUE)
                        {
                           sprintf(&errorparm[strlen(errorparm)], "%c%s (duplicate parameter) ", cmdline[i-1], token) ;
                           clo[j].found = FALSE ;
                        }
                        if (sscanf(parmvalue, "%i", &intval) == 1)
                        {
                           clo[j].value = intval ;
                           clo[j].found = TRUE ;
                        }
                        else
                        { /* we support some special values: off=0, on=1 */
                           if (stricmp(parmvalue,"off") == 0)
                           {
                              clo[j].value = 0 ;
                              clo[j].found = TRUE ;
                           }
                           if (stricmp(parmvalue,"on") == 0)
                           {
                              clo[j].value = 1 ;
                              clo[j].found = TRUE ;
                           }
                        }
                        if (!(clo[j].found))
                           sprintf(&errorparm[strlen(errorparm)], "%c%s (invalid value)", cmdline[i-1], token) ;
                     }
                     if (clo[j].parmtype == PARM_STRING)
                     { /* copy parmvalue to destination */
                        if (clo[j].value != NULL)
                        {
                           strncpy(clo[j].dest, parmvalue, clo[j].value) ;
                           ((_BYTE *) clo[j].dest)[clo[j].value-1] = '\0' ; /* force string termination */
                           clo[j].found = TRUE ;
                        }
                     }
                  }
                  else
                  { /* unexpected parameter */
                     sprintf(&errorparm[strlen(errorparm)], "%c%s ", cmdline[i-1], token) ;
                  }
                  i += strlen(token) ;
               }
            }
            else
            { /* this is a positional parameter (filename e.t.c.) */
               if (wl_get_token(&cmdline[i], &token) == FALSE) cmd_done = TRUE ;
               /* Now we search the next unassigned positional parameter in the list. */
               found = FALSE ;
               for (j=0; (j<clo_size) && (!found); j++)
               {
                  if ((clo[j].parmtype == PARM_POS) && (clo[j].found == FALSE) && (clo[j].dest != NULL))
                  {
                     strncpy(clo[j].dest, token, clo[j].value) ;
                     clo[j].found = TRUE ;
                     found = TRUE ;
                  }
               }
               i += strlen(token) ;
            }
         }
      }
   }
   i = strlen(errorparm) ;
   if (i > 0) i-- ;
   if (errorparm[i] == ' ') errorparm[i] = '\0' ; /* remove trailing blank for the beauty of the error message */
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       get_registry                                                                                                    |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Get the required data from registry. Return TRUE if all keys were successfully read.                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_BOOL get_registry(MYREG *myreg)
{
HKEY keyhandle ;
LONG retv ;
_BOOL rc ;
_LONG csize ;
_WORD i ;
   rc = FALSE ;
   if (!disablereg)
   {
      rc = TRUE ;
// Open Registry key
      retv = RegOpenKeyEx(myreg->hkey,
             myreg->regkeyname,
             0,
             KEY_READ,
             &keyhandle) ;
      if (retv != ERROR_SUCCESS) rc = FALSE ;
      else
      {
         if (retv != ERROR_SUCCESS) rc = FALSE ;
         for (i=0; i<myreg->size ; i++)
         {
            switch (myreg->regvalues[i].valuetype)
            {
               case REG_DWORD :
               csize = 4 ;
               retv = RegQueryValueEx(
                      keyhandle,                         // handle of key to query
                      myreg->regvalues[i].valuename,     // address of name of value to query
                      0,                                 // reserved
                      NULL,                              // address of buffer for value type
                      (LPBYTE)myreg->regvalues[i].value, // address of data buffer
                      &csize                             // address of data buffer size
                      ) ;
               if (retv != ERROR_SUCCESS) rc = FALSE ;
               break ;

               case REG_SZ :
               case REG_MULTI_SZ :
               csize = myreg->regvalues[i].valuesize ;
               retv = RegQueryValueEx(
                      keyhandle,                         // handle of key to query
                      myreg->regvalues[i].valuename,     // address of name of value to query
                      0,                                 // reserved
                      NULL,                              // address of buffer for value type
                      (LPBYTE)myreg->regvalues[i].value, // address of data buffer
                      &csize                             // address of data buffer size
                      ) ;
               if (retv != ERROR_SUCCESS) rc = FALSE ;
               break ;
            }
         }
         if (retv == ERROR_SUCCESS)
         {// Key was found. Do something
         }
         RegCloseKey(keyhandle) ;
      }
   }
   return(rc) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       put_registry                                                                                                    |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Put data into registry                                                                                                |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void put_registry(MYREG *myreg)
{
HKEY keyhandle ;
LONG retv ;
_LONG csize ;
_LONG disposition ;
_WORD i ;

   if (!disablereg)
   {
// Open or create new key
      retv = RegCreateKeyEx(myreg->hkey,       // handle of an open key
             myreg->regkeyname,                   // address of subkey name
             0,                            // reserved
             NULL,                         // address of class string
             REG_OPTION_NON_VOLATILE,      // special options flag
             KEY_ALL_ACCESS,               // desired security access
             REG_OPTION_NON_VOLATILE,      // address of key security structure
             &keyhandle,                   // address of buffer for opened handle
             &disposition                  // address of disposition value buffer
         ) ;
      if (retv == ERROR_SUCCESS)
      { // write data
         for (i=0; i<myreg->size ; i++)
         {
            switch (myreg->regvalues[i].valuetype)
            {
               case REG_DWORD :
               retv = RegSetValueEx(
                      keyhandle,                               // handle of key to set value for
                      myreg->regvalues[i].valuename,           // address of name to set
                      0,                                       // reserved
                      REG_DWORD,                               // flag for value type
                      (CONST BYTE *)myreg->regvalues[i].value, // address of data
                      4                                        // size of value data
                      ) ;
               break ;

               case REG_SZ :
               csize = strlen(myreg->regvalues[i].value) + 1 ;
               retv = RegSetValueEx(
                      keyhandle,                        // handle of key to set value for
                      myreg->regvalues[i].valuename,    // address of value to set
                      0,                                // reserved
                      REG_SZ,                           // flag for value type
                      myreg->regvalues[i].value,        // address of value data
                      csize                             // size of value data
                      ) ;
               break ;

               case REG_MULTI_SZ :
               csize = str2len(myreg->regvalues[i].value) + 2 ;
               retv = RegSetValueEx(
                      keyhandle,                        // handle of key to set value for
                      myreg->regvalues[i].valuename,    // address of value to set
                      0,                                // reserved
                      REG_MULTI_SZ,                     // flag for value type
                      myreg->regvalues[i].value,        // address of value data
                      csize                             // size of value data
                      ) ;
               break ;
            }
         }
         RegCloseKey(keyhandle) ;
      }
   }
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       str2len                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Calculate the length of an array of null-terminated strings, terminated by two null characters (REG_MULTI_SZ).        |
| As with strlen, the size of the two terminating 0's is not counted.                                                   |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
_WORD str2len(_BYTE *str)
{
_WORD i ;
   i = 0 ;
   while ((str[i] != '\0') || (str[i+1] != '\0')) i++ ;
   return(i) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       clear_registry                                                                                                  |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Clear all registry keys which this program has created.                                                               |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void clear_registry(MYREG *myreg)
{
HKEY keyhandle ;
LONG retv ;
   retv = RegOpenKeyEx(myreg->hkey,
          myreg->regkeyname,
          0,
          KEY_WRITE,
          &keyhandle) ;
   if (retv == ERROR_SUCCESS)
   {
      RegDeleteKey(myreg->hkey, myreg->regkeyname) ;
      RegCloseKey(keyhandle) ;
   }
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       void ShowError(int dialog)                                                                                      |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Display last error message on the DLG_STATUS control                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void ShowError(int dialog)
{
_BYTE *lpMsgBuf ;

FormatMessage(
    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
    NULL,
    GetLastError(),
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    (LPTSTR) &lpMsgBuf,
    0,
    NULL) ;
    SetDlgItemText(MainWindow, dialog, lpMsgBuf) ;
// Free the buffer.
    LocalFree(lpMsgBuf) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       set_process_prio                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Set priority of current process.                                                                                      |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void set_process_prio(_SWORD prio)
{
_WORD last = (sizeof(process_prio)/sizeof(process_prio[0])) - 1 ;
   if (prio < 0) prio = 0 ;
   if (prio > last) prio = last ;
   SetPriorityClass(GetCurrentProcess(), process_prio[prio]) ; /* set priority */
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       set_thread_prio                                                                                                 |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Set priority of a thread.                                                                                             |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void set_thread_prio(HANDLE hthread, _SWORD prio)
{
_WORD last = (sizeof(thread_prio)/sizeof(thread_prio[0])) - 1 ;
   if (prio < 0) prio = 0 ;
   if (prio > last) prio = last ;
   SetThreadPriority(hthread, thread_prio[prio]) ; /* set priority */
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       void display_info(_BYTE *msg)                                                                                   |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Display messages on the DLG_INFO control. DLG_INFO is a multi line edit control.                                      |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
//void display_info(_BYTE *msg)
//{
//HWND msg_hwnd ;
//WORD iLen ;
//   msg_hwnd = GetDlgItem(MainWindow, DLG_INFO) ;
//   iLen = GetWindowTextLength(msg_hwnd) ;
//   SendMessage(msg_hwnd, EM_SETSEL, iLen, iLen);
//   SendMessage(msg_hwnd, EM_REPLACESEL, (BOOL) FALSE, (WPARAM) msg) ;
//   iLen = GetWindowTextLength(msg_hwnd) ;
//   SendMessage(msg_hwnd, EM_SETSEL, iLen, iLen);
//   SendMessage(msg_hwnd, EM_REPLACESEL, (BOOL) FALSE, (WPARAM) NEWLINE) ;
//}


