/*
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|          Program Name: WinSample                                                                                      |
|                Author: Michael Krmer                                                                                 |
|                        Michael@dk8pp.de               (private)                                                       |
|                        Michael.Kraemer@eu.necel.com   (business)                                                      |
|                  Date: 30-Dec-2004                                                                                    |
|              Language: Open Watcom C 1.7                                                                              |
|               Version: 0.1                                                                                            |
|        Program status: still testing                                                                                  |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
|  Purpose: Sample skeleton for Windows dialog                                                                          |
|                                                                                                                       |
|  To Do:   Test priorities of program and thread (should work but seems a bit weird)                                   |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|
|   Revision History:
|   -----------------
|   $Log: WinSample.c,v $
|   Revision 1.1  2008/10/12 16:35:17  Meister Lampe
|   First version
|
|   Revision 1.24  2008/02/15 15:35:51  kraemerm
|   break instruction was missing
|
|   Revision 1.23  2008/02/15 15:04:39  kraemerm
|   original dialog size is minimum dialog size
|
|   Revision 1.22  2008/02/13 17:23:03  kraemerm
|   Thread may return a pointer to a message
|
|   Revision 1.21  2008/02/12 16:41:12  kraemerm
|   Cosmetics
|
|   Revision 1.20  2008/02/11 17:09:28  kraemerm
|   Description for command line parameters added.
|   New info dialog box.
|
|   Revision 1.19  2008/02/07 16:17:51  kraemerm
|   Optional Suspend/Proceed/Abort implemented
|
|   Revision 1.18  2008/02/01 15:56:31  kraemerm
|   Operation is moved to a thread
|   Priority setting added (-pp for process, -pt for thread)
|
|   Revision 1.17  2008/01/04 15:17:19  kraemerm
|   small cosmetic changes, program version added to registry
|
|   Revision 1.16  2008/01/03 16:10:09  kraemerm
|   some minor fixes
|
|   Revision 1.15  2007/12/21 15:19:51  kraemerm
|   Command line parameter scanning improved
|
|   Revision 1.14  2007/12/04 16:45:24  kraemerm
|   command line parameter scanning moved to WinLib
|   command line parameters changed to -rc (clear registry) and -rd (disable registry)
|
|   Revision 1.13  2007/12/03 17:09:04  kraemerm
|   Registry support moved out to WinLib
|
|   Revision 1.12  2007/11/30 15:21:40  kraemerm
|   Some Windows functions were exported to new WinLib
|
|   Revision 1.11  2007/11/28 12:58:38  kraemerm
|   implemented -r- switch to avoid using the registry
|
|   Revision 1.10  2007/10/22 13:16:23  kraemerm
|   Info box updated
|   switch to delete registry key implemented (-rd)
|
|   Revision 1.9  2007/02/20 08:07:54  KraemerM
|   New Icon created, registry used to store dialog size and location, include file moved to project directory
|
|   Revision 1.7  2006/06/29 13:10:37  EUR\KraemerM
|   Display of status messages implemented
|   Sample code for setting dialog fonts addes
|
|   Revision 1.6  2006/06/29 12:18:03  EUR\KraemerM
|   Maximize and minimize fixed
|
|   Revision 1.5  2006/06/29 08:28:54  EUR\KraemerM
|   Controls are moved in a sorted way, so that they do not overlap
|   and do not destroy other controls.
|
|   Revision 1.4  2006/06/28 09:32:56  EUR\KraemerM
|   Comments adapted
|
|   Revision 1.3  2006/06/27 16:44:00  EUR\KraemerM
|   Email address in comment adapted
|
|   Revision 1.2  2006/06/27 16:41:57  EUR\KraemerM
|   Resizing a dialog may be limited to a minimum size
|
|
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Includes                                                                                                        |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include <time.h>
#include "WinDef.h"
#include "mytype.h"
#include "WinSample.h"
#include "WinLib.h"

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Defines                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
*/
#define STRINGSIZE 256          /* default size for strings */
#define PARMSIZE 20             /* maximum size of a string or positional command line parameter */
#define SUSPENDBUTTON           /* define if a suspend button shall be supported */

#define MY_HKEY HKEY_CURRENT_USER // handle of registry key
#define MY_REG_KEY MY_REG_KEY_ROOT "WinSample" // Registry key used by this program
#define PROGRAMNAME "Windows Dialog Skeleton"
#define PURPOSE "This program is a sample skeleton for creating Windows programs,\r\r\n"\
                "which consist of just one dialog.\r\r\n"\
                "\r\r\n"
#define PROGRAMVERSION 10       /* will be displayed as n.nn */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       structs and enums                                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
*/

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       global variables                                                                                                |
+-----------------------------------------------------------------------------------------------------------------------+
*/
HINSTANCE       MyInstance ;
HWND            MainWindow ;
_BYTE strng1[STRINGSIZE] ;
_BYTE strng2[STRINGSIZE] ;
/*
 Registry definitions
*/
_SWORD registry_x ;             /* left edge of dialog */
_SWORD registry_y ;             /* top edge of dialog */
_SWORD registry_width ;         /* width of dialog */
_SWORD registry_height ;        /* height of dialog */
_SWORD registry_version ;       /* version of program which wrote the registry */
_BYTE registry_filename[STRINGSIZE+1] ; /* filename for registry */
REGVALUE regvalue[] = {{"posx", REG_DWORD, &registry_x, 0},
                       {"posy", REG_DWORD, &registry_y, 0},
                       {"width", REG_DWORD, &registry_width, 0},
                       {"height", REG_DWORD, &registry_height, 0},
                       {"version", REG_DWORD, &registry_version, 0},
                       {"Filename", REG_SZ, &registry_filename, STRINGSIZE}} ;
MYREG myreg = {MY_HKEY, MY_REG_KEY, &regvalue[0], sizeof(regvalue)/sizeof(regvalue[0])} ;

/*
 Command line parameter definitions
*/
_BOOL disablereg = FALSE ;      /* disable use of the registry */
_BYTE posparm1[PARMSIZE+1] ;    /* positional parameter 1 */
_BYTE posparm2[PARMSIZE+1] ;    /* positional parameter 2 */
_BYTE strngp[PARMSIZE+1] ;      /* string parameter strng */

CLO clo[] = {{"rc", PARM_SIMPLE, NULL, 0, FALSE, "clear registry entries for this program"},
             {"rd", PARM_SIMPLE, NULL, 0, FALSE, "disable use of the registry for this program"},
             {"pp", PARM_INT, NULL, 0, FALSE, "set process priority for dialog\r\r\n"
                                       "       (0:idle, 1:normal, 2:high, 3:realtime)"},
             {"pt", PARM_INT, NULL, 0, FALSE, "set priority for processing thread\r\r\n"
                                       "       (0:idle, 1:lowest, 2:below normal, 3:normal,\r\r\n"
                                       "        4:above normal, 5:highest, 6:time critical)"},
             {"a",  PARM_INT, NULL, 0, FALSE, "test..."},
             {"b",  PARM_INT, NULL, 0, FALSE, "test..."},
             {"strng", PARM_STRING, &strngp, PARMSIZE+1, FALSE, "test string"},
             {NULL, PARM_POS, &posparm1, PARMSIZE+1, FALSE, "test (first positional parameter)"},
             {NULL, PARM_POS, &posparm2, PARMSIZE+1, FALSE, "test (second positional parameter)"}} ;
_WORD clo_size = sizeof(clo)/sizeof(clo[0]) ;

/* These are indexes into the CLO table above. Keep them in sync */
#define PARM_RC 0
#define PARM_RD 1
#define PARM_PP 2
#define PARM_PT 3
#define PARM_A 4
#define PARM_B 5
#define PARM_STRNG 6
#define PARM_POS1 7
#define PARM_POS2 8

_BYTE errorparm[MAXPARMLENGTH] ;

/*
 Dialog coordinates definitions
*/
COORDS coordinates[] = {{0, 0, 0, 0, 0, 0},  /* first entry is reserved for main dialog */
#ifdef SUSPENDBUTTON
                        {BUT_SUSPEND, 0, 0, 0, 0, MOVE_X|MOVE_Y},
#endif
                        {DLG_STATUS, 0, 0, 0, 0, RESIZE_X|RESIZE_Y|MOVE_Y},
                        {DLG_OUTFILENAME, 0, 0, 0, 0, RESIZE_X},
                        {BUT_INFO, 0, 0, 0, 0, MOVE_Y},
                        {BUT_START, 0, 0, 0, 0, MOVE_X|MOVE_Y},
                        {BUT_EXIT, 0, 0, 0, 0, MOVE_X|MOVE_Y}
                       } ;
COORDS *sorted_coord[sizeof(coordinates)/sizeof(coordinates[0])-1] ;
_WORD coord_size = sizeof(coordinates)/sizeof(coordinates[0]) ;
_BOOL validreg = FALSE ;        /* valid data from registry retrieved */
_BOOL abort_thread = FALSE ; /* set TRUE to abort the thread */
_BOOL thread_suspended = FALSE ;
_WORD volatile vol ;
_WORD volatile vol1 ;
_BYTE purpose[] = PURPOSE ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       Function prototypes                                                                                             |
+-----------------------------------------------------------------------------------------------------------------------+
*/
BOOL CALLBACK WinSample(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam) ;
BOOL CALLBACK DlgHelp(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam) ;
DWORD WINAPI DoOperation(LPVOID lpParam) ;
void EnableControls(_BOOL enable) ;

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       WinMain                                                                                                         |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| MAIN program.                                                                                                         |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
int WINAPI WinMain(HINSTANCE this_inst, HINSTANCE prev_inst, LPSTR cmdline, int cmdshow)
{
_LONG static volatile rv ;
_BOOL startup ;
   startup = TRUE ;
   errorparm[0] = '\0' ;
   getparms(cmdline, errorparm) ;
   if (clo[PARM_RC].found) clear_registry(&myreg) ;
   disablereg = clo[PARM_RD].found ;
   if (startup)
   { /* start windowed version */
      if (clo[PARM_PP].found) set_process_prio(clo[PARM_PP].value) ;
      MyInstance = this_inst ;
      InitCommonControls() ;
      rv = DialogBox(this_inst, "DLG_WINSAMPLE", 0, (DLGPROC)WinSample) ;
   }
   return(0);
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       WinSample                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Processes messages for the DLG_WINSAMPLE dialog                                                                       |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
BOOL CALLBACK WinSample(HWND window_handle, unsigned msg, WPARAM wparam, LPARAM lparam)
{
OPENFILENAME outfilestruct ;

//FILE *outfile ;
_BYTE static outfilename[256] = "" ;
_BYTE static outfilename_no_path[256] ;
_BYTE static def_out_ext[] = "CSV files\0*.csv\0All files\0*.*\0" ;
WORD retcd ; /* return nonzero, if the message is processed here and zero, if it is not */
DWORD static dwThreadId, dwThrdParam = 1 ;
HANDLE static hThread ;
RECT rect ;
HFONT hFont ;
_BYTE *out_msg ;

   retcd = 0 ; /* return 0 if we don't process the message */
   switch(msg)
   {
   case WM_INITDIALOG:
      retcd = 1 ;
      MainWindow = window_handle ;
// Make sure to define the below mentioned icon!
      SendMessage(MainWindow, WM_SETICON, ICON_BIG, (LPARAM) LoadIcon(MyInstance, "MyWinIcon")) ;
//    ShowError(DLG_STATUS) ; /* display last error message */

// save original coordinates of dialog
      SaveCoordinates() ;

// Select font for controls (sample code)
      hFont = CreateFont(
          15,                  // logical height of font
          0,                   // logical average character width
          0,                   // angle of escapement
          0,                   // base-line orientation angle
          FW_NORMAL,           // font weight
          FALSE,               // italic attribute flag
          FALSE,               // underline attribute flag
          FALSE,               // strikeout attribute flag
          ANSI_CHARSET,        // character set identifier
          OUT_DEFAULT_PRECIS,  // output precision
          CLIP_DEFAULT_PRECIS, // clipping precision
          DEFAULT_QUALITY,     // output quality
          DEFAULT_PITCH,       // pitch and family
          "Courier New"        // pointer to typeface name string
      ) ;

      SendMessage(GetDlgItem(MainWindow, DLG_OUTFILENAME), WM_SETFONT, (WPARAM) hFont, MAKELPARAM(FALSE, 0));

      CheckRadioButton(MainWindow, RAD_T1, RAD_T4,RAD_T1) ;

      validreg = get_registry(&myreg) ; // get registry content

      if (validreg)
      {
         MoveWindow(
         MainWindow,            // handle of window
         registry_x,            // horizontal position
         registry_y,            // vertical position
         registry_width,        // width
         registry_height,       // height
         TRUE);                 // repaint flag
         strcpy(outfilename, registry_filename) ;
         SetDlgItemText(MainWindow, DLG_OUTFILENAME, outfilename) ;
      }

      /* display any illegal parameter */
      if (strlen(errorparm) != 0)
      {
         LoadString(MyInstance, STR_INVALIDPARM, strng1, STRINGSIZE);
         sprintf(strng2, strng1, errorparm) ;
         SetDlgItemText(MainWindow, DLG_STATUS, strng2) ;
      }
#ifdef SUSPENDBUTTON
      EnableWindow(GetDlgItem(MainWindow, BUT_SUSPEND), FALSE) ;
#endif
   break ;

   case WM_USER:
      retcd = 1 ;
      switch(LOWORD(wparam))
      {
      case USER_THREAD_ENDED:
      // output the results
         LoadString(MyInstance, STR_NORMAL_END, strng1, STRINGSIZE) ;
         out_msg = strng1 ;
/* Thread may have assembled a message. If so, use it */
         if (lparam != NULL) out_msg = (_BYTE *) lparam ;
         SetDlgItemText(MainWindow, DLG_STATUS, out_msg) ;
         CloseHandle(hThread) ;
         hThread = NULL ;
         EnableControls(TRUE) ;
#ifdef SUSPENDBUTTON
         EnableWindow(GetDlgItem(MainWindow, BUT_SUSPEND), FALSE) ;
#endif
      break ;
      }
   break ;

   case WM_COMMAND:
      retcd = 1 ;
      switch(LOWORD(wparam))
      {
      case BUT_OUTPUT:
         outfilestruct.lStructSize = sizeof(OPENFILENAME) ;
         outfilestruct.hwndOwner = MainWindow ;
         outfilestruct.hInstance = 0 ;
         outfilestruct.lpstrFilter = def_out_ext ;
         outfilestruct.lpstrCustomFilter = NULL ;
         outfilestruct.nMaxCustFilter = 0 ;
         outfilestruct.nFilterIndex = 1 ;
         outfilestruct.lpstrFile = outfilename ;
         outfilestruct.nMaxFile = sizeof(outfilename) ;
         outfilestruct.lpstrFileTitle = outfilename_no_path ;
         outfilestruct.nMaxFileTitle = sizeof(outfilename_no_path) ;
         outfilestruct.lpstrInitialDir = NULL ;
         outfilestruct.lpstrTitle = "Sample data output file" ;
         outfilestruct.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT ;
         outfilestruct.nFileOffset = 0 ;
         outfilestruct.nFileExtension = 0 ;
         outfilestruct.lpstrDefExt = NULL ;
         outfilestruct.lCustData = NULL ;
         outfilestruct.lpfnHook = NULL ;
         outfilestruct.lpTemplateName = NULL ;
         if (GetSaveFileName(&outfilestruct) != 0)
         {
            SetDlgItemText(MainWindow, DLG_OUTFILENAME, outfilestruct.lpstrFile) ;
         }
      break ;

      case DLG_OUTFILENAME:
         GetDlgItemText(MainWindow, DLG_OUTFILENAME, outfilename, sizeof(outfilename)) ;
      break ;

      case BUT_INFO:
         sprintf(strng1, "Version %u.%2.2u", PROGRAMVERSION/100, PROGRAMVERSION%100) ;
         PrintProgramInfo(DLG_HELPLIST, PROGRAMNAME, strng1, __DATE__, __TIME__) ;
      break ;

      case RAD_T1:
         CheckRadioButton(MainWindow, RAD_T1,RAD_T4,RAD_T1) ;
      break ;

#ifdef SUSPENDBUTTON
      case BUT_SUSPEND:
         if (!thread_suspended)
         {
            if (SuspendThread(hThread) != -1) thread_suspended = TRUE ;
            SetDlgItemText(MainWindow, BUT_SUSPEND, "Resume") ;
            LoadString(MyInstance, STR_SUSPENDED, strng1, STRINGSIZE) ;
            SetDlgItemText(MainWindow, DLG_STATUS, strng1) ;
            LoadString(MyInstance, STR_BUTABORT, strng1, STRINGSIZE) ;
            SetDlgItemText(MainWindow, BUT_START, strng1) ;
            EnableWindow(GetDlgItem(MainWindow, BUT_START), TRUE) ;
         }
         else
         {
            if (ResumeThread(hThread) != -1) thread_suspended = FALSE ;
            SetDlgItemText(MainWindow, BUT_SUSPEND, "Suspend") ;
            LoadString(MyInstance, STR_WORKING, strng1, STRINGSIZE) ;
            SetDlgItemText(MainWindow, DLG_STATUS, strng1) ;
            LoadString(MyInstance, STR_BUTSTART, strng1, STRINGSIZE) ;
            SetDlgItemText(MainWindow, BUT_START, strng1) ;
            EnableWindow(GetDlgItem(MainWindow, BUT_START), FALSE) ;
         }
      break ;
#endif

      case IDOK:
      case BUT_START:
         /* The start button becomes abort, if the thread is suspended */
         if (thread_suspended)
         {
            abort_thread = TRUE ;
#ifdef SUSPENDBUTTON
            SendMessage(MainWindow, WM_COMMAND, BUT_SUSPEND, 0) ;
#endif
         }
         else
         {
/* Operation is done in a thread. Start it. */
            hThread = CreateThread(
                NULL,                        // no security attributes
                0,                           // use default stack size
                DoOperation,                 // thread function
                &dwThrdParam,                // argument to thread function
                0,                           // use default creation flags
                &dwThreadId);                // returns the thread identifier

        // Check the return value for success.
            if (hThread == NULL)
               LoadString(MyInstance, STR_THREADSTARTERROR, strng1, STRINGSIZE) ;
            else
            { /* Thread started properly */
               LoadString(MyInstance, STR_WORKING, strng1, STRINGSIZE) ;
               EnableControls(FALSE) ;
               thread_suspended = FALSE ;
#ifdef SUSPENDBUTTON
               EnableWindow(GetDlgItem(MainWindow, BUT_SUSPEND), TRUE) ;
#endif
               if (clo[PARM_PT].found) set_thread_prio(hThread, clo[PARM_PT].value) ;
            }
            SetDlgItemText(MainWindow, DLG_STATUS, strng1) ;
         }
      break ;

      case IDABORT:
      case IDCANCEL:
      case BUT_EXIT:
         GetWindowRect(MainWindow, &rect) ; /* get current dialog coordinates */
         registry_x = rect.left ;
         registry_y = rect.top ;
         registry_width = rect.right - rect.left ;
         registry_height = rect.bottom - rect.top ;
         if (registry_x < 0) registry_x = 0 ;
         if (registry_y < 0) registry_y = 0 ;
         strcpy(registry_filename, outfilename) ;
         registry_version = PROGRAMVERSION ;    /* version of program which wrote the registry */
         put_registry(&myreg) ;                 /* store the dialog properties in registry */
         EndDialog(MainWindow, TRUE) ;          /* exits the dialog box */
      break ;

      default:
         retcd = 0 ; /* we have not processed this command, so return zero */
      break ;
      }
   break ;

   default:
      retcd = WinLibMessage(msg, wparam, lparam) ; /* WinLib processes some messages */
   break ;
   }

   return(retcd) ;
} /* WinSample */

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       void EnableControls(_BOOL enable)                                                                               |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| Enable or disable some critical controls during execution of thread.                                                  |
| Control gets enabled if enable is TRUE, otherwise it gets disabled.                                                   |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
void EnableControls(_BOOL enable)
{
   EnableWindow(GetDlgItem(MainWindow, BUT_START), enable) ;
   EnableWindow(GetDlgItem(MainWindow, RAD_T1), enable) ;
   EnableWindow(GetDlgItem(MainWindow, RAD_T2), enable) ;
   EnableWindow(GetDlgItem(MainWindow, RAD_T3), enable) ;
   EnableWindow(GetDlgItem(MainWindow, RAD_T4), enable) ;
   EnableWindow(GetDlgItem(MainWindow, BUT_OUTPUT), enable) ;
}

/*
+-----------------------------------------------------------------------------------------------------------------------+
|       DoOperation(LPVOID lpParam)                                                                                     |
+-----------------------------------------------------------------------------------------------------------------------+
|                                                                                                                       |
| This thread does whatever this program is supposed to do.                                                             |
|                                                                                                                       |
+-----------------------------------------------------------------------------------------------------------------------+
*/
DWORD WINAPI DoOperation(LPVOID lpParam)
{
_BYTE static strng_thread1[STRINGSIZE] ;
_BYTE *retcd ;

   retcd = NULL ; /* NULL means no error */

   for (vol1=0;((vol1<10) && !abort_thread);vol1++)
      for (vol=0;vol<500000000;vol++) ;
//   Sleep(15000) ; /* in msec */
   if (abort_thread)
   {
      LoadString(MyInstance, STR_ABORTED, strng_thread1, STRINGSIZE) ;
      abort_thread = FALSE ;
   }
   else
      LoadString(MyInstance, STR_SUCCESS, strng_thread1, STRINGSIZE);
   retcd = strng_thread1 ;

// Notify main window that the thread is finished
   SendMessage(MainWindow, WM_USER, USER_THREAD_ENDED, (LPARAM) retcd) ;

   return 0;
}


